/**
 * Theme Name: errigal
 * Theme URI: https://projecterrigal.com/themes/errigal/
 * Author: Project errigal
 * Author URI: https://projecterrigal.com/
 * Version: 1.0.4
 */
( function() {
	'use strict';

	const CHARACTER_PERIOD = '.';
	const CLASS_MOBILE_SEARCH_BACKGROUND = 'mobile-search';
	const CLASS_NAV_MAIN_NAVIGATION = 'main-navigation';
	const CLASS_NAV_MENU_CONTAINER = 'menu-container';
	const CLASS_SECTION_SITE = 'site';
	const CLASS_SECTION_SITE_HEADER = 'site-header';
	const EMPTY_STRING = '';
	const EVENT_TYPE_LOAD = 'load';
	const EVENT_TYPE_RESIZE = 'resize';
	const EVENT_TYPE_SCROLL = 'scroll';
	const ID_ADMIN_BAR = 'wpadminbar';
	const NODE_TYPE_HTML_ELEMENT = 1;
	const NODE_TYPE_DOCUMENT = 9;
	const POSITION_FIXED = 'fixed';
	const UNIT_PIXEL = 'px';

	function matches( element, selectorString, isTag ) {
		if ( ! isHTMLElement( element ) || ! isString( selectorString ) || isNullOrEmpty( selectorString ) ) {
			return false;
		}

		if ( ! sanitiseBoolean( isTag ) && selectorString.indexOf( CHARACTER_PERIOD ) !== 0 ) {
			selectorString = CHARACTER_PERIOD + selectorString;
		}

		return element.matches ? element.matches( selectorString ) : element.msMatchesSelector( selectorString );
	}

	function getElementByClassName( className, rootElement ) {
		if ( ! isString( className ) || isNullOrEmpty( className ) ) {
			return null;
		}

		if ( ! rootElement ) {
			rootElement = document;
		}

		return isHTMLElement( rootElement ) ? rootElement.getElementsByClassName( className )[ 0 ] : null;
	}

	function isFixed( element ) {
		if ( ! isHTMLElement( element ) ) {
			return false;
		}

		return window.getComputedStyle( element, null ).position.toLowerCase() === POSITION_FIXED;
	}

	function sanitiseBoolean( value ) {
		return isBoolean( value ) ? value : false;
	}

	function isBoolean( value ) {
		return value === false || value === true;
	}

	function isHTMLElement( element, className ) {
		if ( ! element ) {
			return false;
		}

		const IS_HTML_ELEMENT = element.nodeType === NODE_TYPE_HTML_ELEMENT && element instanceof HTMLElement;
		const IS_DOCUMENT = element.nodeType === NODE_TYPE_DOCUMENT && element instanceof Document;
		const IS_HTML_ELEMENT_OR_DOCUMENT = IS_HTML_ELEMENT || IS_DOCUMENT;

		if ( isString( className ) && ! isNullOrEmpty( className ) ) {
			return IS_HTML_ELEMENT_OR_DOCUMENT && matches( element, className );
		}

		return IS_HTML_ELEMENT_OR_DOCUMENT;
	}

	function isNullOrEmpty( value ) {
		return value === null || value === undefined || value === EMPTY_STRING;
	}

	function isString( value ) {
		return typeof value === 'string';
	}

	function getAdminBar() {
		return document.getElementById( ID_ADMIN_BAR );
	}

	function getMainNavigation() {
		return getElementByClassName( CLASS_NAV_MAIN_NAVIGATION );
	}

	function getMenuContainer() {
		return getElementByClassName( CLASS_NAV_MENU_CONTAINER );
	}

	function getMobileSearchBackground() {
		return getElementByClassName( CLASS_MOBILE_SEARCH_BACKGROUND );
	}

	function getSite() {
		return getElementByClassName( CLASS_SECTION_SITE );
	}

	function getSiteHeader() {
		return getElementByClassName( CLASS_SECTION_SITE_HEADER );
	}

	function addFixedElementPositionEvents() {
		window.addEventListener( EVENT_TYPE_LOAD, () => {
			adjustFixedElementPosition();
			adjustSiteMinHeight();
		} );
		window.addEventListener( EVENT_TYPE_RESIZE, () => {
			adjustFixedElementPosition();
			adjustSiteMinHeight();
		} );
		window.addEventListener( EVENT_TYPE_SCROLL, () => {
			adjustFixedElementPosition();
		} );
	}

	function adjustFixedElementPosition() {
		const ADMIN_BAR = getAdminBar();

		if ( ! ADMIN_BAR ) {
			return;
		}

		const ADMIN_BAR_BOTTOM_POSITION = ADMIN_BAR.getBoundingClientRect().bottom;
		const ADMIN_BAR_HEIGHT = ( ADMIN_BAR_BOTTOM_POSITION > 0 ? ADMIN_BAR_BOTTOM_POSITION : 0 ) + UNIT_PIXEL;
		const SITE_HEADER = getSiteHeader();

		if ( SITE_HEADER ) {
			SITE_HEADER.style.top = isFixed( SITE_HEADER ) ? ADMIN_BAR_HEIGHT : null;
		}

		const MAIN_NAVIGATION = getMainNavigation();

		if ( MAIN_NAVIGATION ) {
			MAIN_NAVIGATION.style.marginTop = isFixed( MAIN_NAVIGATION ) ? ADMIN_BAR_HEIGHT : null;
		}

		const MOBILE_MENU = getMenuContainer();

		if ( MOBILE_MENU ) {
			MOBILE_MENU.style.marginTop = isFixed( MOBILE_MENU ) ? ADMIN_BAR_HEIGHT : null;
		}

		const MOBILE_SEARCH = getMobileSearchBackground();

		if ( MOBILE_SEARCH ) {
			MOBILE_SEARCH.style.marginTop = isFixed( MOBILE_SEARCH ) ? ADMIN_BAR_HEIGHT : null;
		}
	}

	function adjustSiteMinHeight() {
		const ADMIN_BAR = getAdminBar();
		const SITE = getSite();

		if ( ADMIN_BAR && SITE ) {
			const ADMIN_BAR_BOTTOM_POSITION = ADMIN_BAR.getBoundingClientRect().bottom;
			const ADMIN_BAR_HEIGHT = ADMIN_BAR_BOTTOM_POSITION > 0 ? ADMIN_BAR_BOTTOM_POSITION : 0;

			SITE.style.minHeight = window.innerHeight - ADMIN_BAR_HEIGHT + UNIT_PIXEL;
		}
	}

	addFixedElementPositionEvents();
}() );
