/**
 * Theme Name: errigal
 * Theme URI: https://projecterrigal.com/themes/errigal/
 * Author: Project errigal
 * Author URI: https://projecterrigal.com/
 * Version: 1.0.4
 */
( function() {
	'use strict';

	const ADMIN_DISMISS_ADMIN_NOTICE_WELCOME = 'errigal_dismiss_admin_notice_welcome';
	const ATTRIBUTE_NAME_VALUE = 'value';
	const CHARACTER_PERIOD = '.';
	const CLASS_ADMIN_NOTICE_DISMISS = 'notice-dismiss';
	const CLASS_WIDGET_ADD_IMAGE_BUTTON_CONTAINER = 'errigal-widget__add-image-button-container';
	const CLASS_WIDGET_ADD_IMAGE_BUTTON = 'errigal-widget__add-image-button';
	const CLASS_WIDGET_IMAGE = 'errigal-widget__image';
	const CLASS_WIDGET_IMAGE_BUTTONS_CONTAINER = 'errigal-widget__image-buttons-container';
	const CLASS_WIDGET_IMAGE_CONTAINER = 'errigal-widget__image-container';
	const CLASS_WIDGET_IMAGE_PATH = 'errigal-widget__image-path';
	const CLASS_WIDGET_REMOVE_IMAGE_BUTTON = 'errigal-widget__remove-image-button';
	const CLASS_WIDGET_REPLACE_IMAGE_BUTTON = 'errigal-widget__replace-image-button';
	const EMPTY_STRING = '';
	const EMPTY_VALUE = '';
	const EVENT_TYPE_CHANGE = 'change';
	const EVENT_TYPE_CLICK = 'click';
	const EVENT_TYPE_EVENT = 'Event';
	const EVENT_TYPE_SELECT = 'select';
	const ID_ADMIN_NOTICE_WELCOME = 'errigal-welcome-notice';
	const MEDIA_STATE_SELECTION = 'selection';
	const MEDIA_TYPE_IMAGE = 'image';
	const NODE_TYPE_HTML_ELEMENT = 1;
	const NODE_TYPE_DOCUMENT = 9;
	const WIDGET_IMAGE_BUTTON_STYLE_DISPLAY_BLOCK = 'block';
	const WIDGET_IMAGE_BUTTON_STYLE_DISPLAY_NONE = 'none';

	function addAdminNoticeEvents() {
		// eslint-disable-next-line no-undef
		jQuery( document ).ready( ( $ ) => {
			$( `#${ ID_ADMIN_NOTICE_WELCOME } .${ CLASS_ADMIN_NOTICE_DISMISS }` ).click( () => {
				// eslint-disable-next-line no-undef
				$.post( ajaxurl, {
					action: ADMIN_DISMISS_ADMIN_NOTICE_WELCOME,
					// eslint-disable-next-line no-undef
					nonce: ERRIGAL_ADMIN.nonce,
				} );
			} );
		} );
	}

	function getMediaUploader() {
		// eslint-disable-next-line no-undef
		return wp.media( {
			// eslint-disable-next-line no-undef
			title: ERRIGAL_ADMIN.admin_media_modal_title,
			library: {
				type: MEDIA_TYPE_IMAGE,
			},
			button: {
				// eslint-disable-next-line no-undef
				text: ERRIGAL_ADMIN.admin_media_modal_button_text_select,
			},
			multiple: false,
		} );
	}

	function matches( element, selectorString, isTag ) {
		if ( ! isHTMLElement( element ) || ! isString( selectorString ) || isNullOrEmpty( selectorString ) ) {
			return false;
		}

		if ( ! sanitiseBoolean( isTag ) && selectorString.indexOf( CHARACTER_PERIOD ) !== 0 ) {
			selectorString = CHARACTER_PERIOD + selectorString;
		}

		return element.matches ? element.matches( selectorString ) : element.msMatchesSelector( selectorString );
	}

	function getElementByClassName( className, rootElement ) {
		if ( ! isString( className ) || isNullOrEmpty( className ) ) {
			return null;
		}

		if ( ! rootElement ) {
			rootElement = document;
		}

		return isHTMLElement( rootElement ) ? rootElement.getElementsByClassName( className )[ 0 ] : null;
	}

	function isIe() {
		return /(trident|msie)/i.test( navigator.userAgent );
	}

	function sanitiseBoolean( value ) {
		return isBoolean( value ) ? value : false;
	}

	function isBoolean( value ) {
		return value === false || value === true;
	}

	function isHTMLElement( element, className ) {
		if ( ! element ) {
			return false;
		}

		const IS_HTML_ELEMENT = element.nodeType === NODE_TYPE_HTML_ELEMENT && element instanceof HTMLElement;
		const IS_DOCUMENT = element.nodeType === NODE_TYPE_DOCUMENT && element instanceof Document;
		const IS_HTML_ELEMENT_OR_DOCUMENT = IS_HTML_ELEMENT || IS_DOCUMENT;

		if ( isString( className ) && ! isNullOrEmpty( className ) ) {
			return IS_HTML_ELEMENT_OR_DOCUMENT && matches( element, className );
		}

		return IS_HTML_ELEMENT_OR_DOCUMENT;
	}

	function isNullOrEmpty( value ) {
		return value === null || value === undefined || value === EMPTY_STRING;
	}

	function isString( value ) {
		return typeof value === 'string';
	}

	function addAdminWidgetImageEvents() {
		document.addEventListener( EVENT_TYPE_CLICK, ( event ) => {
			if ( isWidgetAddImageButton( event.target ) || isWidgetReplaceImageButton( event.target ) ) {
				openMediaUploader( event.target );

				return;
			}

			if ( isWidgetRemoveImageButton( event.target ) ) {
				updateWidgetImage( event.target, EMPTY_VALUE );
			}
		} );
	}

	function createChangeEvent() {
		if ( isIe() ) {
			const EVENT = document.createEvent( EVENT_TYPE_EVENT );

			EVENT.initEvent( EVENT_TYPE_CHANGE, true, true );

			return EVENT;
		}

		return new Event(
			EVENT_TYPE_CHANGE,
			{ bubbles: true }
		);
	}

	function isWidgetAddImageButton( element ) {
		return isHTMLElement( element, CLASS_WIDGET_ADD_IMAGE_BUTTON );
	}

	function isWidgetRemoveImageButton( element ) {
		return isHTMLElement( element, CLASS_WIDGET_REMOVE_IMAGE_BUTTON );
	}

	function isWidgetReplaceImageButton( element ) {
		return isHTMLElement( element, CLASS_WIDGET_REPLACE_IMAGE_BUTTON );
	}

	function openMediaUploader( button ) {
		if ( ! isHTMLElement( button ) ) {
			return;
		}

		const MEDIA_UPLOADER = getMediaUploader();

		if ( MEDIA_UPLOADER ) {
			MEDIA_UPLOADER.on( EVENT_TYPE_SELECT, () => {
				const ATTACHMENT = MEDIA_UPLOADER.state().get( MEDIA_STATE_SELECTION ).first().toJSON();
				const IMAGE_PATH = ATTACHMENT && ATTACHMENT.url ? ATTACHMENT.url : null;

				if ( isString( IMAGE_PATH ) ) {
					updateWidgetImage( button, IMAGE_PATH );
				}
			} );
			MEDIA_UPLOADER.open();
		}
	}

	function toggleAddImageButtonAndReplaceRemoveButtons( button, widgetImageControlParent ) {
		if ( ! isHTMLElement( button ) || ! isHTMLElement( widgetImageControlParent ) ) {
			return;
		}

		const WIDGET_IMAGE_CONTAINER = getElementByClassName( CLASS_WIDGET_IMAGE_CONTAINER, widgetImageControlParent );
		const WIDGET_ADD_IMAGE_BUTTON_CONTAINER = getElementByClassName( CLASS_WIDGET_ADD_IMAGE_BUTTON_CONTAINER, widgetImageControlParent );
		const WIDGET_IMAGE_BUTTONS_CONTAINER = getElementByClassName( CLASS_WIDGET_IMAGE_BUTTONS_CONTAINER, widgetImageControlParent );

		if ( WIDGET_IMAGE_CONTAINER && WIDGET_ADD_IMAGE_BUTTON_CONTAINER && WIDGET_IMAGE_BUTTONS_CONTAINER ) {
			if ( matches( button, CLASS_WIDGET_ADD_IMAGE_BUTTON ) ) {
				WIDGET_IMAGE_CONTAINER.style.display = EMPTY_VALUE;
				WIDGET_ADD_IMAGE_BUTTON_CONTAINER.style.display = WIDGET_IMAGE_BUTTON_STYLE_DISPLAY_NONE;
				WIDGET_IMAGE_BUTTONS_CONTAINER.style.display = WIDGET_IMAGE_BUTTON_STYLE_DISPLAY_BLOCK;

				return;
			}

			if ( matches( button, CLASS_WIDGET_REMOVE_IMAGE_BUTTON ) ) {
				WIDGET_IMAGE_CONTAINER.style.display = WIDGET_IMAGE_BUTTON_STYLE_DISPLAY_NONE;
				WIDGET_ADD_IMAGE_BUTTON_CONTAINER.style.display = WIDGET_IMAGE_BUTTON_STYLE_DISPLAY_BLOCK;
				WIDGET_IMAGE_BUTTONS_CONTAINER.style.display = WIDGET_IMAGE_BUTTON_STYLE_DISPLAY_NONE;
			}
		}
	}

	function updateWidgetImage( button, imagePath ) {
		if ( ! isHTMLElement( button ) || ! isString( imagePath ) ) {
			return;
		}

		const WIDGET_IMAGE_BUTTON_CONTAINER = button.parentElement;
		const WIDGET_IMAGE_CONTROL = WIDGET_IMAGE_BUTTON_CONTAINER ? WIDGET_IMAGE_BUTTON_CONTAINER.parentElement : undefined;
		const WIDGET_IMAGE = WIDGET_IMAGE_CONTROL ? getElementByClassName( CLASS_WIDGET_IMAGE, WIDGET_IMAGE_CONTROL ) : undefined;
		const WIDGET_IMAGE_PATH = WIDGET_IMAGE_CONTROL ? getElementByClassName( CLASS_WIDGET_IMAGE_PATH, WIDGET_IMAGE_CONTROL ) : undefined;

		if ( WIDGET_IMAGE_CONTROL && WIDGET_IMAGE && WIDGET_IMAGE_PATH && WIDGET_IMAGE_PATH.hasAttribute( ATTRIBUTE_NAME_VALUE ) ) {
			if ( isIe() ) {
				WIDGET_IMAGE.style.backgroundImage = `url('${ imagePath }')`;
			} else {
				WIDGET_IMAGE.src = imagePath;
			}

			WIDGET_IMAGE_PATH.value = imagePath;

			const CHANGE_EVENT = createChangeEvent();

			WIDGET_IMAGE_PATH.dispatchEvent( CHANGE_EVENT );
			toggleAddImageButtonAndReplaceRemoveButtons( button, WIDGET_IMAGE_CONTROL );
		}
	}

	addAdminNoticeEvents();
	addAdminWidgetImageEvents();
}() );
