/**
 * Theme Name: errigal
 * Theme URI: https://projecterrigal.com/themes/errigal/
 * Author: Project errigal
 * Author URI: https://projecterrigal.com/
 * Version: 1.0.4
 */
( function() {
	'use strict';

	const ATTRIBUTE_NAME_HREF = 'href';
	const CHARACTER_PERIOD = '.';
	const CLASS_CONTENT_HOMEPAGE_TITLE = 'homepage-title';
	const CLASS_CUSTOMISER_PREFIX_SITE_HEADER = 'site-header-';
	const CLASS_SECTION_SITE_HEADER = 'site-header';
	const CUSTOMISER_ID_BLOGNAME = 'blogname';
	const CUSTOMISER_ID_COLOUR_SCHEME = 'colour_scheme';
	const CUSTOMISER_ID_SITE_HEADER_COLOUR = 'site_header_colour';
	const CUSTOMISER_ID_TYPOGRAPHY_BASE_FONT = 'base_font';
	const CUSTOMISER_ID_TYPOGRAPHY_FONT_SIZE = 'font_size';
	const CUSTOMISER_ID_TYPOGRAPHY_HEADING_FONT = 'heading_font';
	const CUSTOM_TYPOGRAPHY_GOOGLE_FONT_FAMILY_URL = '//fonts.googleapis.com/css?family=';
	const EMPTY_STRING = '';
	const EVENT_TYPE_CLICK_PREVIEW = 'click.preview';
	const EVENT_TYPE_PREVIEW_READY = 'preview-ready';
	const FONT_FALLBACK = '-apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, Oxygen-Sans, Ubuntu, Cantarell, "Helvetica Neue", sans-serif';
	const ID_CSS_ERRIGAL_FONT_STYLE_BASE = 'errigal-base-google-font-style-css';
	const ID_CSS_ERRIGAL_FONT_STYLE_HEADING = 'errigal-heading-google-font-style-css';
	const ID_CSS_ERRIGAL_STYLE = 'errigal-style-css';
	const ID_CSS_ERRIGAL_STYLE_INLINE = 'errigal-style-inline-css';
	const NODE_TYPE_HTML_ELEMENT = 1;
	const NODE_TYPE_DOCUMENT = 9;
	const SELECTOR_A_HREF = 'a[href]';
	const TAG_BODY = 'body';
	const TAG_HYPERLINK = 'a';

	function addClass( element, className ) {
		if ( isHTMLElement( element ) && element.classList && isString( className ) && ! isNullOrEmpty( className ) ) {
			element.classList.add( className );
		}
	}

	function matches( element, selectorString, isTag ) {
		if ( ! isHTMLElement( element ) || ! isString( selectorString ) || isNullOrEmpty( selectorString ) ) {
			return false;
		}

		if ( ! sanitiseBoolean( isTag ) && selectorString.indexOf( CHARACTER_PERIOD ) !== 0 ) {
			selectorString = CHARACTER_PERIOD + selectorString;
		}

		return element.matches ? element.matches( selectorString ) : element.msMatchesSelector( selectorString );
	}

	function removeClass( element, className ) {
		if ( isHTMLElement( element ) && element.classList && isString( className ) && ! isNullOrEmpty( className ) ) {
			element.classList.remove( className );
		}
	}

	function getElementByClassName( className, rootElement ) {
		if ( ! isString( className ) || isNullOrEmpty( className ) ) {
			return null;
		}

		if ( ! rootElement ) {
			rootElement = document;
		}

		return isHTMLElement( rootElement ) ? rootElement.getElementsByClassName( className )[ 0 ] : null;
	}

	function sanitiseBoolean( value ) {
		return isBoolean( value ) ? value : false;
	}

	function isBoolean( value ) {
		return value === false || value === true;
	}

	function isHTMLElement( element, className ) {
		if ( ! element ) {
			return false;
		}

		const IS_HTML_ELEMENT = element.nodeType === NODE_TYPE_HTML_ELEMENT && element instanceof HTMLElement;
		const IS_DOCUMENT = element.nodeType === NODE_TYPE_DOCUMENT && element instanceof Document;
		const IS_HTML_ELEMENT_OR_DOCUMENT = IS_HTML_ELEMENT || IS_DOCUMENT;

		if ( isString( className ) && ! isNullOrEmpty( className ) ) {
			return IS_HTML_ELEMENT_OR_DOCUMENT && matches( element, className );
		}

		return IS_HTML_ELEMENT_OR_DOCUMENT;
	}

	function isLinkElement( element ) {
		return Boolean( isHTMLElement( element ) && element.nodeName && element.nodeName.toLowerCase() === TAG_HYPERLINK );
	}

	function isNullOrEmpty( value ) {
		return value === null || value === undefined || value === EMPTY_STRING;
	}

	function isString( value ) {
		return typeof value === 'string';
	}

	function getCssErrigalFontStyleBase() {
		return document.getElementById( ID_CSS_ERRIGAL_FONT_STYLE_BASE );
	}

	function getCssErrigalFontStyleHeading() {
		return document.getElementById( ID_CSS_ERRIGAL_FONT_STYLE_HEADING );
	}

	function getCssErrigalStyle() {
		return document.getElementById( ID_CSS_ERRIGAL_STYLE );
	}

	function getCssErrigalStyleInline() {
		return document.getElementById( ID_CSS_ERRIGAL_STYLE_INLINE );
	}

	function getSiteHeader() {
		return getElementByClassName( CLASS_SECTION_SITE_HEADER );
	}

	function addColourSchemeEvents() {
		// eslint-disable-next-line no-undef
		wp.customize( CUSTOMISER_ID_COLOUR_SCHEME, ( value ) => {
			value.bind( ( colourScheme ) => {
				const CSS_ERRIGAL_STYLE = getCssErrigalStyle();

				if ( CSS_ERRIGAL_STYLE ) {
					const CSS_FILE_NAME = isNullOrEmpty( colourScheme ) ? 'style.min.css' : `style-${ colourScheme }.min.css`;

					CSS_ERRIGAL_STYLE.setAttribute( ATTRIBUTE_NAME_HREF, CSS_ERRIGAL_STYLE.getAttribute( ATTRIBUTE_NAME_HREF ).replace( /style.*.min.css/, CSS_FILE_NAME ) );
				}
			} );
		} );
	}

	function addLinkEvents() {
		// eslint-disable-next-line no-undef
		const BODY = jQuery( TAG_BODY );

		BODY.off( EVENT_TYPE_CLICK_PREVIEW );
		BODY.on( EVENT_TYPE_CLICK_PREVIEW, SELECTOR_A_HREF, ( event ) => {
			if ( isLinkToExternalSite( event.target ) ) {
				event.preventDefault();
			}
		} );
	}

	function isLinkToExternalSite( element ) {
		return isLinkElement( element ) && element.host !== window.location.host;
	}

	function addSiteHeaderColourEvents() {
		// eslint-disable-next-line no-undef
		wp.customize( CUSTOMISER_ID_SITE_HEADER_COLOUR, ( value ) => {
			value.bind( ( siteHeaderColour ) => {
				const SITE_HEADER = getSiteHeader();

				if ( SITE_HEADER ) {
					for ( let i = 0; i < SITE_HEADER.classList.length; i++ ) {
						const CLASS_NAME = SITE_HEADER.classList[ i ];

						if ( CLASS_NAME.includes( CLASS_CUSTOMISER_PREFIX_SITE_HEADER ) ) {
							SITE_HEADER.classList.remove( CLASS_NAME );
							SITE_HEADER.classList.add( CLASS_CUSTOMISER_PREFIX_SITE_HEADER + siteHeaderColour );

							return;
						}
					}
				}
			} );
		} );
	}

	function addStaticPageBlognameEvents() {
		// eslint-disable-next-line no-undef
		wp.customize( CUSTOMISER_ID_BLOGNAME, ( setting ) => {
			setting.bind( ( blogname ) => {
				const HOMEPAGE_TITLE = getElementByClassName( CLASS_CONTENT_HOMEPAGE_TITLE );

				if ( HOMEPAGE_TITLE ) {
					HOMEPAGE_TITLE.innerHTML = blogname;
				}
			} );
		} );
	}

	function addTypographyEvents() {
		const CSS_ERRIGAL_STYLE_INLINE = getCssErrigalStyleInline();

		// eslint-disable-next-line no-undef
		wp.customize( CUSTOMISER_ID_TYPOGRAPHY_BASE_FONT, ( value ) => {
			value.bind( ( fontName ) => {
				// eslint-disable-next-line no-undef
				const FONT_NAME = ERRIGAL_FONT_NAMES[ fontName ];
				// eslint-disable-next-line no-undef
				const FONT_FAMILY = ERRIGAL_FONT_FAMILIES[ fontName ];
				const CSS_ERRIGAL_FONT_STYLE_BASE = getCssErrigalFontStyleBase();

				if ( FONT_FAMILY && CSS_ERRIGAL_FONT_STYLE_BASE && CSS_ERRIGAL_STYLE_INLINE ) {
					CSS_ERRIGAL_FONT_STYLE_BASE.setAttribute( ATTRIBUTE_NAME_HREF, `${ CUSTOM_TYPOGRAPHY_GOOGLE_FONT_FAMILY_URL }${ FONT_FAMILY }` );
					CSS_ERRIGAL_STYLE_INLINE.sheet.cssRules[ 0 ].style.fontFamily = `"${ FONT_NAME }", ${ FONT_FALLBACK }`;
				}
			} );
		} );

		// eslint-disable-next-line no-undef
		wp.customize( CUSTOMISER_ID_TYPOGRAPHY_HEADING_FONT, ( value ) => {
			value.bind( ( fontName ) => {
				// eslint-disable-next-line no-undef
				const FONT_NAME = ERRIGAL_FONT_NAMES[ fontName ];
				// eslint-disable-next-line no-undef
				const FONT_FAMILY = ERRIGAL_FONT_FAMILIES[ fontName ];
				const CSS_ERRIGAL_FONT_STYLE_HEADING = getCssErrigalFontStyleHeading();

				if ( FONT_FAMILY && CSS_ERRIGAL_FONT_STYLE_HEADING && CSS_ERRIGAL_STYLE_INLINE ) {
					CSS_ERRIGAL_FONT_STYLE_HEADING.setAttribute( ATTRIBUTE_NAME_HREF, `${ CUSTOM_TYPOGRAPHY_GOOGLE_FONT_FAMILY_URL }${ FONT_FAMILY }` );
					CSS_ERRIGAL_STYLE_INLINE.sheet.cssRules[ 1 ].style.fontFamily = `"${ FONT_NAME }", ${ FONT_FALLBACK }`;
				}
			} );
		} );

		// eslint-disable-next-line no-undef
		wp.customize( CUSTOMISER_ID_TYPOGRAPHY_FONT_SIZE, ( value ) => {
			value.bind( ( fontSize ) => {
				// eslint-disable-next-line no-undef
				const FONT_SIZES = ERRIGAL_FONT_SIZES;

				if ( fontSize && FONT_SIZES && FONT_SIZES.includes( fontSize ) ) {
					for ( let i = 0; i < FONT_SIZES.length; i++ ) {
						removeClass( document.documentElement, FONT_SIZES[ i ] );
					}

					addClass( document.documentElement, fontSize );
				}
			} );
		} );
	}

	// eslint-disable-next-line no-undef
	wp.customize.bind( EVENT_TYPE_PREVIEW_READY, () => {
		addColourSchemeEvents();
		addLinkEvents();
		addSiteHeaderColourEvents();
		addStaticPageBlognameEvents();
		addTypographyEvents();
	} );
}() );
