/**
 * Theme Name: errigal
 * Theme URI: https://projecterrigal.com/themes/errigal/
 * Author: Project errigal
 * Author URI: https://projecterrigal.com/
 * Version: 1.0.4
 */
( function() {
	'use strict';

	const ATTRIBUTE_NAME_TABINDEX = 'tabIndex';
	const ATTRIBUTE_VALUE_MINUS_ONE = '-1';
	const CALLBACK_INITIAL_ATTEMPT_COUNT = 0;
	const CALLBACK_INITIAL_INTERVAL = 0;
	const CALLBACK_INTERVAL = 50;
	const CALLBACK_MAX_ATTEMPT_COUNT = 20;
	const CHARACTER_HASH = '#';
	const CHARACTER_PERIOD = '.';
	const CLASS_COMPONENT_INPUT_FIELD = 'input-field';
	const CLASS_CONTENT_PAGE_NUMBERS = 'page-numbers';
	const CLASS_HEADER_SEARCH = 'site-header__search';
	const CLASS_HEADER_SEARCH_BACKGROUND = 'site-header__search-background';
	const CLASS_HEADER_SEARCH_BUTTON = 'site-header__search-button';
	const CLASS_HEADER_SEARCH_CLOSE_BUTTON = 'site-header__close-search-button';
	const CLASS_NAV_BACK_TO_TOP = 'back-to-top';
	const CLASS_NAV_BOTTOM_NAVBAR = 'bottom-navbar';
	const CLASS_NAV_MAIN_NAVIGATION = 'main-navigation';
	const CLASS_NAV_MENU_WITH_SUB_MENU = 'menu-item-has-children';
	const CLASS_NAV_SKIP_LINK = 'skip-link';
	const CLASS_SEARCH_FORM_CLEAR_BUTTON = 'search-clear-button';
	const CLASS_SEARCH_FORM_FIELD = 'search-field';
	const CLASS_SEARCH_FORM_SUBMIT = 'search-submit';
	const CLASS_SECTION_SITE_FOOTER = 'site-footer';
	const CLASS_SECTION_SITE_HEADER = 'site-header';
	const CLASS_SHARED_ACTIVE = 'active';
	const CLASS_SHARED_BOTTOM = 'bottom';
	const CLASS_SHARED_SHOW = 'show';
	const CLASS_SHARED_SHOW_V_SCROLL = 'show-v-scroll';
	const CLASS_SHARED_STICKY = 'sticky';
	const CLASS_WP_BLOCK_EMBED_YOUTUBE = 'wp-block-embed-youtube';
	const DISTANCE_REM_MD = 1.5;
	const DISTANCE_REM_LG = 2;
	const EMPTY_STRING = '';
	const EMPTY_VALUE = '';
	const EVENT_TYPE_CLICK = 'click';
	const EVENT_TYPE_FOCUSIN = 'focusin';
	const EVENT_TYPE_FOCUSOUT = 'focusout';
	const EVENT_TYPE_INPUT = 'input';
	const EVENT_TYPE_KEYDOWN = 'keydown';
	const EVENT_TYPE_LOAD = 'load';
	const EVENT_TYPE_RESIZE = 'resize';
	const EVENT_TYPE_SCROLL = 'scroll';
	const KEY_ENTER = [ 'Enter', '13' ];
	const KEY_ESCAPE = [ 'Escape', '27' ];
	const KEY_TAB = [ 'Tab', '9' ];
	const KEY_ID_KEY = 0;
	const KEY_ID_KEYCODE = 1;
	const NODE_TYPE_HTML_ELEMENT = 1;
	const NODE_TYPE_DOCUMENT = 9;
	const POSITION_FIXED = 'fixed';
	const SCROLL_DISTANCE_SHORT = 200;
	const SCROLL_DISTANCE_MODERATE = 1000;
	const SCROLL_DISTANCE_LONG = 2000;
	const SCROLL_DURATION_SWIFT = 50;
	const SCROLL_DURATION_MODERATE = 200;
	const SCROLL_DURATION_NORMAL = 300;
	const SCROLL_DURATION_SLOW = 400;
	const TAG_HYPERLINK = 'a';
	const TAG_IFRAME = 'iframe';
	const TAG_INPUT = 'input';
	const TAG_LABEL = 'label';
	const TAG_TEXTAREA = 'textarea';
	const TAG_TIME = 'time';
	const TAGS_FOCUSABLE_ELEMENTS = [ 'A', 'BUTTON', 'INPUT', 'SELECT', 'TEXTAREA' ];
	const UNIT_PIXEL = 'px';

	function focusElementCallback( element, interval, attemptCount ) {
		if ( ! isHTMLElement( element ) || isNaN( interval ) || isNaN( attemptCount ) || attemptCount === CALLBACK_MAX_ATTEMPT_COUNT ) {
			return;
		}

		setTimeout( () => {
			element.focus();

			if ( element === document.activeElement ) {
				return;
			}

			attemptCount++;
			focusElementCallback( element, CALLBACK_INTERVAL, attemptCount );
		}, interval );
	}

	function getIframes() {
		return document.getElementsByTagName( TAG_IFRAME );
	}

	function getInputFields() {
		return document.querySelectorAll( `.${ CLASS_COMPONENT_INPUT_FIELD } > ${ TAG_INPUT }, .${ CLASS_COMPONENT_INPUT_FIELD } > ${ TAG_TEXTAREA }` );
	}

	function getBackToTopButton() {
		return getElementByClassName( CLASS_NAV_BACK_TO_TOP );
	}

	function getBottomNavbar() {
		return getElementByClassName( CLASS_NAV_BOTTOM_NAVBAR );
	}

	function getMainNavigation() {
		return getElementByClassName( CLASS_NAV_MAIN_NAVIGATION );
	}

	function getHeaderSearch() {
		return getElementByClassName( CLASS_HEADER_SEARCH );
	}

	function getHeaderSearchBackground() {
		return getElementByClassName( CLASS_HEADER_SEARCH_BACKGROUND );
	}

	function getHeaderSearchButton() {
		return getElementByClassName( CLASS_HEADER_SEARCH_BUTTON );
	}

	function getHeaderSearchCloseButton() {
		return getElementByClassName( CLASS_HEADER_SEARCH_CLOSE_BUTTON );
	}

	function getHeaderSearchField() {
		const HEADER_SEARCH = getHeaderSearch();

		return HEADER_SEARCH ? getElementByClassName( CLASS_SEARCH_FORM_FIELD, HEADER_SEARCH ) : null;
	}

	function getSiteFooter() {
		return getElementByClassName( CLASS_SECTION_SITE_FOOTER );
	}

	function getSiteHeader() {
		return getElementByClassName( CLASS_SECTION_SITE_HEADER );
	}

	function getSkipLink() {
		return getElementByClassName( CLASS_NAV_SKIP_LINK );
	}

	function getPositionY( element ) {
		if ( ! isHTMLElement( element ) ) {
			return 0;
		}

		const TARGET_ELEMENT_OFFSET_HEIGHT = getElementTop( element );

		if ( element === document.body || TARGET_ELEMENT_OFFSET_HEIGHT === 0 ) {
			return TARGET_ELEMENT_OFFSET_HEIGHT;
		}

		const FIXED_ELEMENT_HEIGHT = getFixedElementHeight();
		const TOP_MARGIN = convertRemToPx( DISTANCE_REM_LG );

		return TARGET_ELEMENT_OFFSET_HEIGHT - FIXED_ELEMENT_HEIGHT - TOP_MARGIN;
	}

	function scrollCallback( animationTime, element, startTime, duration ) {
		if ( ! isNumber( animationTime ) || ! isHTMLElement( element ) || ! isNumber( startTime ) || ! isNumber( duration ) ) {
			return;
		}

		const CURRENT_Y = document.documentElement.scrollTop;
		const DESTINATION_Y = getPositionY( element );
		const PROGRESS = animationTime - startTime;
		const TIME = Math.min( 1, PROGRESS / duration );

		if ( PROGRESS < duration ) {
			// eslint-disable-next-line no-mixed-operators
			window.scrollTo( 0, CURRENT_Y + ( DESTINATION_Y - CURRENT_Y ) * TIME );
			requestAnimationFrame( ( currentTime ) => {
				scrollCallback( currentTime, element, startTime, duration );
			} );

			return;
		}

		window.scrollTo( 0, DESTINATION_Y );

		if ( ! isFocusableElement( element ) ) {
			element.setAttribute( ATTRIBUTE_NAME_TABINDEX, ATTRIBUTE_VALUE_MINUS_ONE );
		}

		focusScrollTargetElement( element );
	}

	function focusScrollTargetElement( element ) {
		if ( ! isHTMLElement( element ) ) {
			return;
		}

		if ( ! isFocusableElement( element ) ) {
			element.setAttribute( ATTRIBUTE_NAME_TABINDEX, ATTRIBUTE_VALUE_MINUS_ONE );
		}

		const SCROLL_POSITION = {
			x: window.pageXOffset || document.documentElement.scrollLeft,
			y: window.pageYOffset || document.documentElement.scrollTop,
		};

		element.focus();
		window.scrollTo( SCROLL_POSITION.x, SCROLL_POSITION.y );
	}

	function getElementTop( element ) {
		if ( ! isHTMLElement( element ) ) {
			return 0;
		}

		const RECT = element.getBoundingClientRect();
		const SCROLL_TOP = window.pageYOffset || document.documentElement.scrollTop;

		return RECT.top + SCROLL_TOP;
	}

	function getFixedElementHeight() {
		const SITE_HEADER = getSiteHeader();

		if ( SITE_HEADER && isFixed( SITE_HEADER ) ) {
			return SITE_HEADER.getBoundingClientRect().bottom;
		}

		const STICKY_ELEMENTS_IN_SITE_HEADER = SITE_HEADER && SITE_HEADER.getElementsByClassName( CLASS_SHARED_STICKY );

		if ( STICKY_ELEMENTS_IN_SITE_HEADER && STICKY_ELEMENTS_IN_SITE_HEADER.length !== 0 ) {
			return STICKY_ELEMENTS_IN_SITE_HEADER[ STICKY_ELEMENTS_IN_SITE_HEADER.length - 1 ].getBoundingClientRect().bottom;
		}

		return 0;
	}

	function addClass( element, className ) {
		if ( isHTMLElement( element ) && element.classList && isString( className ) && ! isNullOrEmpty( className ) ) {
			element.classList.add( className );
		}
	}

	function matches( element, selectorString, isTag ) {
		if ( ! isHTMLElement( element ) || ! isString( selectorString ) || isNullOrEmpty( selectorString ) ) {
			return false;
		}

		if ( ! sanitiseBoolean( isTag ) && selectorString.indexOf( CHARACTER_PERIOD ) !== 0 ) {
			selectorString = CHARACTER_PERIOD + selectorString;
		}

		return element.matches ? element.matches( selectorString ) : element.msMatchesSelector( selectorString );
	}

	function removeClass( element, className ) {
		if ( isHTMLElement( element ) && element.classList && isString( className ) && ! isNullOrEmpty( className ) ) {
			element.classList.remove( className );
		}
	}

	function getElementByClassName( className, rootElement ) {
		if ( ! isString( className ) || isNullOrEmpty( className ) ) {
			return null;
		}

		if ( ! rootElement ) {
			rootElement = document;
		}

		return isHTMLElement( rootElement ) ? rootElement.getElementsByClassName( className )[ 0 ] : null;
	}

	function getElementByTagName( qualifiedName, rootElement ) {
		if ( ! isString( qualifiedName ) || isNullOrEmpty( qualifiedName ) ) {
			return null;
		}

		if ( ! rootElement ) {
			rootElement = document;
		}

		return isHTMLElement( rootElement ) ? rootElement.getElementsByTagName( qualifiedName )[ 0 ] : null;
	}

	function convertRemToPx( rem ) {
		if ( ! parseFloat( rem ) ) {
			return 0;
		}

		return parseFloat( getComputedStyle( document.documentElement ).fontSize ) * rem;
	}

	function focusElement( element ) {
		if ( ! isHTMLElement( element ) ) {
			return;
		}

		// eslint-disable-next-line prefer-const
		let attemptCount = CALLBACK_INITIAL_ATTEMPT_COUNT;

		focusElementCallback( element, CALLBACK_INITIAL_INTERVAL, attemptCount );
	}

	function isDescendantOf( element, parentElement ) {
		if ( ! isHTMLElement( element ) || ! isHTMLElement( parentElement ) ) {
			return false;
		}

		let target = element;

		while ( target.parentElement ) {
			if ( parentElement === target.parentElement ) {
				return true;
			}

			target = target.parentElement;
		}

		return false;
	}

	function isFixed( element ) {
		if ( ! isHTMLElement( element ) ) {
			return false;
		}

		return window.getComputedStyle( element, null ).position.toLowerCase() === POSITION_FIXED;
	}

	function scrollTo( element, duration ) {
		if ( ! isHTMLElement( element ) ) {
			return;
		}

		if ( ! duration ) {
			const CURRENT_Y = document.documentElement.scrollTop;
			const DESTINATION_Y = getPositionY( element );
			const DISTANCE = Math.abs( CURRENT_Y - DESTINATION_Y );

			switch ( true ) {
				case DISTANCE < SCROLL_DISTANCE_SHORT:
					duration = SCROLL_DURATION_SWIFT;
					break;
				case DISTANCE < SCROLL_DISTANCE_MODERATE:
					duration = SCROLL_DURATION_MODERATE;
					break;
				case DISTANCE < SCROLL_DISTANCE_LONG:
					duration = SCROLL_DURATION_NORMAL;
					break;
				default:
					duration = SCROLL_DURATION_SLOW;
			}
		}

		const START_TIME = performance.now();

		requestAnimationFrame( ( currentTime ) => {
			scrollCallback( currentTime, element, START_TIME, parseInt( duration ) );
		} );
	}

	function isKey( event, key ) {
		if ( ! isEvent( event ) || ! isStringPair( key ) ) {
			return false;
		}

		if ( isIe() ) {
			return event.keyCode === parseInt( key[ KEY_ID_KEYCODE ] );
		}

		return event.key === key[ KEY_ID_KEY ];
	}

	function isIe() {
		return /(trident|msie)/i.test( navigator.userAgent );
	}

	function sanitiseBoolean( value ) {
		return isBoolean( value ) ? value : false;
	}

	function isBoolean( value ) {
		return value === false || value === true;
	}

	function isEvent( event ) {
		return /\[object (?:\w+)?Event\]/.test( Object.prototype.toString.call( event ) ) && 'type' in event && 'target' in event;
	}

	function isFocusableElement( element ) {
		return isHTMLElement( element ) && TAGS_FOCUSABLE_ELEMENTS.indexOf( element.tagName ) >= 0;
	}

	function isHTMLCollection( element ) {
		return element instanceof HTMLCollection;
	}

	function isHTMLElement( element, className ) {
		if ( ! element ) {
			return false;
		}

		const IS_HTML_ELEMENT = element.nodeType === NODE_TYPE_HTML_ELEMENT && element instanceof HTMLElement;
		const IS_DOCUMENT = element.nodeType === NODE_TYPE_DOCUMENT && element instanceof Document;
		const IS_HTML_ELEMENT_OR_DOCUMENT = IS_HTML_ELEMENT || IS_DOCUMENT;

		if ( isString( className ) && ! isNullOrEmpty( className ) ) {
			return IS_HTML_ELEMENT_OR_DOCUMENT && matches( element, className );
		}

		return IS_HTML_ELEMENT_OR_DOCUMENT;
	}

	function isLinkElement( element ) {
		return Boolean( isHTMLElement( element ) && element.nodeName && element.nodeName.toLowerCase() === TAG_HYPERLINK );
	}

	function isNullOrEmpty( value ) {
		return value === null || value === undefined || value === EMPTY_STRING;
	}

	function isNumber( value ) {
		return typeof value === 'number' && isFinite( value );
	}

	function isString( value ) {
		return typeof value === 'string';
	}

	function isStringPair( value ) {
		if ( ! Array.isArray( value ) || value.length !== 2 ) {
			return false;
		}

		for ( let i = 0; i < value.length; i++ ) {
			if ( ! isString( value[ i ] ) || value[ i ].length === 0 ) {
				return false;
			}
		}

		return true;
	}

	function addAccessibilityCommonEvents() {
		document.addEventListener( EVENT_TYPE_FOCUSOUT, ( event ) => {
			if ( hasMinusTabindex( event.target ) ) {
				removeTabIndex( event.target );
			}
		} );
	}

	function hasMinusTabindex( element ) {
		return isHTMLElement( element ) && element.getAttribute( ATTRIBUTE_NAME_TABINDEX ) === ATTRIBUTE_VALUE_MINUS_ONE;
	}

	function removeTabIndex( element ) {
		if ( isHTMLElement( element ) ) {
			element.removeAttribute( ATTRIBUTE_NAME_TABINDEX );
		}
	}

	function addInputFieldEvents() {
		document.addEventListener( EVENT_TYPE_FOCUSIN, ( event ) => {
			if ( isInputField( event.target ) ) {
				moveInputFieldNameToLegend( event.target );
			}
		} );
		document.addEventListener( EVENT_TYPE_FOCUSOUT, ( event ) => {
			if ( isInputField( event.target ) && event.target.value !== null && event.target.value === EMPTY_VALUE ) {
				moveLegendToInputField( event.target );
			}
		} );
		window.addEventListener( EVENT_TYPE_LOAD, () => {
			initialiseInputFieldLabelPositions();
		} );
	}

	function initialiseInputFieldLabelPositions() {
		const INPUT_FIELDS = getInputFields();

		for ( let i = 0; i < INPUT_FIELDS.length; i++ ) {
			if ( ! isNullOrEmpty( INPUT_FIELDS[ i ].value ) ) {
				moveInputFieldNameToLegend( INPUT_FIELDS[ i ] );
			}
		}
	}

	function getLabel( element ) {
		if ( ! isHTMLElement( element ) || ! isInputField( element ) ) {
			return null;
		}

		const INPUT_FIELD_CONTAINER = element.parentElement;

		return INPUT_FIELD_CONTAINER ? getElementByTagName( TAG_LABEL, INPUT_FIELD_CONTAINER ) : null;
	}

	function isInputField( element ) {
		if ( ! isHTMLElement( element ) || ! ( matches( element, TAG_INPUT, true ) || matches( element, TAG_TEXTAREA, true ) ) ) {
			return false;
		}

		return isHTMLElement( element.parentElement, CLASS_COMPONENT_INPUT_FIELD );
	}

	function moveInputFieldNameToLegend( element ) {
		if ( ! isHTMLElement( element ) || ! isInputField( element ) ) {
			return;
		}

		const LABEL = getLabel( element );

		if ( LABEL ) {
			addClass( LABEL, CLASS_SHARED_ACTIVE );
		}
	}

	function moveLegendToInputField( element ) {
		if ( ! isHTMLElement( element ) || ! isInputField( element ) ) {
			return;
		}

		const LABEL = getLabel( element );

		if ( LABEL ) {
			removeClass( LABEL, CLASS_SHARED_ACTIVE );
		}
	}

	function isInMainNavigation( element ) {
		if ( ! isHTMLElement( element ) ) {
			return false;
		}

		const MAIN_NAVIGATION = getMainNavigation();

		return isDescendantOf( element, MAIN_NAVIGATION );
	}

	function isLinkOfMenuItemWithSubMenu( element ) {
		if ( ! isLinkElement( element ) ) {
			return false;
		}

		return isHTMLElement( element.parentElement, CLASS_NAV_MENU_WITH_SUB_MENU );
	}

	function addAnchorLinkEvents() {
		document.addEventListener( EVENT_TYPE_CLICK, ( event ) => {
			if ( isAnchorLink( event.target ) ) {
				event.preventDefault();
				scrollToAnchorLinkDestination( event.target );
			}
		} );
		document.addEventListener( EVENT_TYPE_KEYDOWN, ( event ) => {
			if ( isKey( event, KEY_ENTER ) ) {
				if ( isAnchorLink( event.target ) ) {
					event.preventDefault();
					scrollToAnchorLinkDestination( event.target );
				}
			}
		} );
		window.addEventListener( EVENT_TYPE_LOAD, () => {
			if ( ! window.location.hash ) {
				return;
			}

			const ANCHOR_LINK_DESTINATION = getAnchorLinkDestination( window.location );

			if ( ! ANCHOR_LINK_DESTINATION ) {
				return;
			}

			if ( isIe() ) {
				// eslint-disable-next-line prefer-const
				let attemptCount = CALLBACK_INITIAL_ATTEMPT_COUNT;

				ensureAnchorLinkDestinationIsVisibleCallback( ANCHOR_LINK_DESTINATION, attemptCount );

				return;
			}

			ensureAnchorLinkDestinationIsVisible( ANCHOR_LINK_DESTINATION );
		} );
	}

	function ensureAnchorLinkDestinationIsVisible( anchorLinkDestination ) {
		if ( isHTMLElement( anchorLinkDestination ) ) {
			scrollTo( anchorLinkDestination, 0 );
		}
	}

	function ensureAnchorLinkDestinationIsVisibleCallback( anchorLinkDestination, attemptCount ) {
		if ( ! anchorLinkDestination || isNaN( attemptCount ) || attemptCount === CALLBACK_MAX_ATTEMPT_COUNT ) {
			return;
		}

		setTimeout( () => {
			scrollTo( anchorLinkDestination, 0 );

			if ( getPositionY( anchorLinkDestination ) === document.documentElement.scrollTop ) {
				return;
			}

			attemptCount++;
			ensureAnchorLinkDestinationIsVisibleCallback( anchorLinkDestination, attemptCount );
		}, CALLBACK_INTERVAL );
	}

	function getAnchorLinkDestination( linkOrLocation ) {
		if ( ! linkOrLocation || ! hasAnchorLink( linkOrLocation ) ) {
			return null;
		}

		if ( isHashOnly( linkOrLocation ) ) {
			return document.body;
		}

		const ID = linkOrLocation.hash.substring( 1 );

		return document.getElementById( ID );
	}

	function getLinkElement( element ) {
		if ( ! isHTMLElement( element ) ) {
			return null;
		}

		const LINK_CANDIDATE_ELEMENT = matches( element, TAG_TIME, true ) ? element.parentElement : element;

		return LINK_CANDIDATE_ELEMENT && matches( LINK_CANDIDATE_ELEMENT, TAG_HYPERLINK, true ) ? LINK_CANDIDATE_ELEMENT : null;
	}

	function hasAnchorLink( linkOrLocation ) {
		return Boolean( linkOrLocation && ( linkOrLocation.hash || isHashOnly( linkOrLocation ) ) );
	}

	function isAnchorLink( element ) {
		if ( ! isHTMLElement( element ) ) {
			return false;
		}

		const LINK_ELEMENT = getLinkElement( element );

		return isLocalAnchorLink( LINK_ELEMENT ) && ! isPaginationNavLink( element ) && ! ( isLinkOfMenuItemWithSubMenu( element ) && isInMainNavigation( element ) );
	}

	function isAnotherLocalAnchorLink( element ) {
		return Boolean( isLinkElement( element ) && element.pathname && element.pathname === window.location.pathname );
	}

	function isHashOnly( linkOrLocation ) {
		return Boolean( linkOrLocation && linkOrLocation.href && linkOrLocation.href.charAt( linkOrLocation.href.length - 1 ) === CHARACTER_HASH );
	}

	function isLocalAnchorLink( element ) {
		if ( ! isLinkElement( element ) ) {
			return false;
		}

		const ANCHOR_LINK_DESTINATION = getAnchorLinkDestination( element );

		return hasAnchorLink( element ) && ( isHTMLElement( ANCHOR_LINK_DESTINATION ) || isAnotherLocalAnchorLink( element ) );
	}

	function isPaginationNavLink( element ) {
		return isLinkElement( element ) && matches( element, CLASS_CONTENT_PAGE_NUMBERS );
	}

	function scrollToAnchorLinkDestination( element ) {
		if ( ! isHTMLElement( element ) ) {
			return;
		}

		const LINK_ELEMENT = getLinkElement( element );

		if ( LINK_ELEMENT ) {
			const ANCHOR_LINK_DESTINATION = getAnchorLinkDestination( LINK_ELEMENT );

			if ( isHTMLElement( ANCHOR_LINK_DESTINATION ) ) {
				scrollTo( ANCHOR_LINK_DESTINATION );
			}
		}
	}

	function addEmbeddedIframeEvents() {
		setIframeSizes();
		window.addEventListener( EVENT_TYPE_LOAD, () => {
			setIframeSizes();
		} );
		window.addEventListener( EVENT_TYPE_RESIZE, () => {
			setIframeSizes();
		} );
	}

	function calculateAspectRatio( width, height ) {
		if ( isNaN( width ) || isNaN( height ) ) {
			return null;
		}

		return parseFloat( width ) / parseFloat( height );
	}

	function getIframeHeight( iframe ) {
		if ( ! shouldResizeIframe( iframe ) || isNaN( iframe.width ) || isNaN( iframe.height ) ) {
			return;
		}

		const PARENT_WIDTH = getParentWidth( iframe );
		const ASPECT_RATIO = calculateAspectRatio( iframe.width, iframe.height );

		if ( ! isNumber( PARENT_WIDTH ) || ! isNumber( ASPECT_RATIO ) ) {
			return 0;
		}

		return Math.round( PARENT_WIDTH / ASPECT_RATIO );
	}

	function getParentWidth( element ) {
		if ( ! isHTMLElement( element ) || ! isHTMLElement( element.parentElement ) ) {
			return null;
		}

		const WIDTH = window.getComputedStyle( element.parentElement, null ).width;

		if ( WIDTH === null || isNaN( parseFloat( WIDTH ) ) ) {
			return null;
		}

		return parseFloat( WIDTH );
	}

	function isIframe( element ) {
		return isHTMLElement( element ) && element.tagName.toLowerCase() === TAG_IFRAME;
	}

	function isLegacyYouTubeIframe( element ) {
		if ( ! isIframe( element ) ) {
			return false;
		}

		const WRAPPER = element.parentElement;
		const FIGURE = WRAPPER ? WRAPPER.parentElement : null;

		if ( ! FIGURE ) {
			return false;
		}

		return isHTMLElement( FIGURE, CLASS_WP_BLOCK_EMBED_YOUTUBE );
	}

	function processIframes( iframes ) {
		if ( ! isHTMLCollection( iframes ) ) {
			return;
		}

		for ( let i = 0; i < iframes.length; i++ ) {
			const IFRAME = iframes[ i ];

			setIframeSize( IFRAME );
		}
	}

	function setIframeSize( iframe ) {
		if ( ! shouldResizeIframe( iframe ) || isNaN( iframe.width ) || isNaN( iframe.height ) ) {
			return;
		}

		const IFRAME_HEIGHT = getIframeHeight( iframe );

		if ( isNumber( IFRAME_HEIGHT ) && IFRAME_HEIGHT > 0 ) {
			iframe.style.width = '100%';
			iframe.style.height = `${ IFRAME_HEIGHT }${ UNIT_PIXEL }`;
		}
	}

	function setIframeSizes() {
		const IFRAMES = getIframes();

		processIframes( IFRAMES );
	}

	function shouldResizeIframe( iframe ) {
		return isIframe( iframe ) && ! isLegacyYouTubeIframe( iframe );
	}

	function addBackToTopEvents() {
		document.addEventListener( EVENT_TYPE_CLICK, ( event ) => {
			if ( isBackToTopButton( event.target ) ) {
				backToTop();
			}
		} );
		document.addEventListener( EVENT_TYPE_KEYDOWN, ( event ) => {
			if ( isKey( event, KEY_ENTER ) ) {
				if ( isBackToTopButton( event.target ) ) {
					event.preventDefault();
					backToTopAndFocusSkipLink();
				}
			}
		} );
		window.addEventListener( EVENT_TYPE_LOAD, () => {
			toggleBackToTopButtonVisibilityAndBottomPosition();
		} );
		window.addEventListener( EVENT_TYPE_SCROLL, () => {
			toggleBackToTopButtonVisibilityAndBottomPosition();
		} );
	}

	function backToTop() {
		scrollTo( document.documentElement );
	}

	function backToTopAndFocusSkipLink() {
		const SKIP_LINK = getSkipLink();

		if ( isLinkElement( SKIP_LINK ) ) {
			scrollTo( SKIP_LINK );

			return;
		}

		backToTop();
	}

	function isBackToTopButton( element ) {
		return isHTMLElement( element, CLASS_NAV_BACK_TO_TOP );
	}

	function setBackToTopBottom() {
		const BACK_TO_TOP_BUTTON = getBackToTopButton();
		const BOTTOM_NAVBAR = getBottomNavbar();

		if ( BACK_TO_TOP_BUTTON && BOTTOM_NAVBAR ) {
			const BACK_TO_TOP_BOTTOM_MARGIN = convertRemToPx( DISTANCE_REM_MD );
			const BOTTOM = BOTTOM_NAVBAR ? BOTTOM_NAVBAR.offsetHeight + BACK_TO_TOP_BOTTOM_MARGIN : BACK_TO_TOP_BOTTOM_MARGIN;

			BACK_TO_TOP_BUTTON.style.bottom = BOTTOM + UNIT_PIXEL;
		}
	}

	function toggleBackToTopButtonBottomPosition() {
		const BACK_TO_TOP_BUTTON = getBackToTopButton();
		const SITE_FOOTER = getSiteFooter();

		if ( BACK_TO_TOP_BUTTON && SITE_FOOTER ) {
			const WINDOW_BOTTOM_POSITION = window.pageYOffset + window.innerHeight;
			const BOTTOM_NAVBAR = getBottomNavbar();
			const FOOTER_TOP = BOTTOM_NAVBAR ? SITE_FOOTER.offsetTop + BOTTOM_NAVBAR.offsetHeight : SITE_FOOTER.offsetTop;

			if ( WINDOW_BOTTOM_POSITION > FOOTER_TOP ) {
				addClass( BACK_TO_TOP_BUTTON, CLASS_SHARED_BOTTOM );
				BACK_TO_TOP_BUTTON.style.bottom = null;

				return;
			}

			removeClass( BACK_TO_TOP_BUTTON, CLASS_SHARED_BOTTOM );
			setBackToTopBottom();
		}
	}

	function toggleBackToTopButtonVisibility() {
		const BACK_TO_TOP_BUTTON = getBackToTopButton();

		if ( BACK_TO_TOP_BUTTON ) {
			const BACK_TO_TOP_BUTTON_SHOW_AT = convertRemToPx( DISTANCE_REM_MD );

			if ( window.pageYOffset > BACK_TO_TOP_BUTTON_SHOW_AT ) {
				addClass( BACK_TO_TOP_BUTTON, CLASS_SHARED_SHOW );

				return;
			}

			removeClass( BACK_TO_TOP_BUTTON, CLASS_SHARED_SHOW );
		}
	}

	function toggleBackToTopButtonVisibilityAndBottomPosition() {
		toggleBackToTopButtonVisibility();
		toggleBackToTopButtonBottomPosition();
	}

	function addHeaderSearchEvents() {
		document.addEventListener( EVENT_TYPE_CLICK, ( event ) => {
			if ( isHeaderSearchButton( event.target ) ) {
				openHeaderSearch();
				focusHeaderSearchField();

				return;
			}

			if ( isHeaderSearchBackground( event.target ) || isHeaderSearchCloseButton( event.target ) ) {
				closeHeaderSearch();
				focusHeaderSearchButton();

				return;
			}

			if ( isHeaderSearchSubmit( event.target ) && headerSearchFieldHasNoKeyword() ) {
				focusHeaderSearchField();
			}
		} );
		document.addEventListener( EVENT_TYPE_KEYDOWN, ( event ) => {
			if ( isKey( event, KEY_ENTER ) ) {
				if ( isHeaderSearchButton( event.target ) ) {
					event.preventDefault();
					openHeaderSearch();
					focusHeaderSearchField();

					return;
				}

				if ( isHeaderSearchCloseButton( event.target ) ) {
					event.preventDefault();
					closeHeaderSearch();
					focusHeaderSearchButton();

					return;
				}

				if ( isHeaderSearchSubmit( event.target ) && headerSearchFieldHasNoKeyword() ) {
					event.preventDefault();
					event.target.click();
					focusHeaderSearchField();

					return;
				}
			}

			if ( isKey( event, KEY_ESCAPE ) ) {
				if ( headerSearchIsOpen() ) {
					event.preventDefault();
					closeHeaderSearch();
					focusHeaderSearchButton();

					return;
				}
			}

			if ( isKey( event, KEY_TAB ) ) {
				if ( event.shiftKey && isHeaderSearchField( event.target ) ) {
					focusHeaderSearchCloseButton();

					return;
				}

				if ( ! event.shiftKey && isHeaderSearchCloseButton( event.target ) ) {
					focusHeaderSearchField();
				}
			}
		} );
	}

	function closeHeaderSearch() {
		const HEADER_SEARCH = getHeaderSearch();
		const HEADER_SEARCH_BACKGROUND = getHeaderSearchBackground();

		if ( HEADER_SEARCH && HEADER_SEARCH_BACKGROUND ) {
			removeClass( document.documentElement, CLASS_SHARED_SHOW_V_SCROLL );
			removeClass( HEADER_SEARCH, CLASS_SHARED_SHOW );
			removeClass( HEADER_SEARCH_BACKGROUND, CLASS_SHARED_SHOW );
		}
	}

	function focusHeaderSearchButton() {
		const HEADER_SEARCH_BUTTON = getHeaderSearchButton();

		if ( HEADER_SEARCH_BUTTON ) {
			focusElement( HEADER_SEARCH_BUTTON );
		}
	}

	function focusHeaderSearchCloseButton() {
		const HEADER_SEARCH_CLOSE_BUTTON = getHeaderSearchCloseButton();

		if ( HEADER_SEARCH_CLOSE_BUTTON ) {
			focusElement( HEADER_SEARCH_CLOSE_BUTTON );
		}
	}

	function focusHeaderSearchField() {
		const HEADER_SEARCH_FIELD = getHeaderSearchField();

		if ( HEADER_SEARCH_FIELD ) {
			focusElement( HEADER_SEARCH_FIELD );
		}
	}

	function headerSearchFieldHasNoKeyword() {
		const HEADER_SEARCH_FIELD = getHeaderSearchField();

		return isHTMLElement( HEADER_SEARCH_FIELD ) && HEADER_SEARCH_FIELD.value !== undefined && HEADER_SEARCH_FIELD.value.length === 0;
	}

	function headerSearchIsOpen() {
		const HEADER_SEARCH = getHeaderSearch();

		return isHTMLElement( HEADER_SEARCH, CLASS_SHARED_SHOW );
	}

	function isHeaderSearchBackground( element ) {
		return isHTMLElement( element, CLASS_HEADER_SEARCH_BACKGROUND );
	}

	function isHeaderSearchButton( element ) {
		return isHTMLElement( element, CLASS_HEADER_SEARCH_BUTTON );
	}

	function isHeaderSearchCloseButton( element ) {
		return isHTMLElement( element, CLASS_HEADER_SEARCH_CLOSE_BUTTON );
	}

	function isHeaderSearchField( element ) {
		if ( ! isHTMLElement( element, CLASS_SEARCH_FORM_FIELD ) ) {
			return false;
		}

		const HEADER_SEARCH = getHeaderSearch();

		return isDescendantOf( element, HEADER_SEARCH );
	}

	function isHeaderSearchSubmit( element ) {
		if ( ! isHTMLElement( element, CLASS_SEARCH_FORM_SUBMIT ) ) {
			return false;
		}

		const HEADER_SEARCH = getHeaderSearch();

		return isDescendantOf( element, HEADER_SEARCH );
	}

	function openHeaderSearch() {
		const HEADER_SEARCH = getHeaderSearch();
		const HEADER_SEARCH_BACKGROUND = getHeaderSearchBackground();

		if ( HEADER_SEARCH && HEADER_SEARCH_BACKGROUND ) {
			addClass( document.documentElement, CLASS_SHARED_SHOW_V_SCROLL );
			addClass( HEADER_SEARCH, CLASS_SHARED_SHOW );
			addClass( HEADER_SEARCH_BACKGROUND, CLASS_SHARED_SHOW );
		}
	}

	function addSearchFormEvents() {
		document.addEventListener( EVENT_TYPE_CLICK, ( event ) => {
			if ( isSearchClearButton( event.target ) ) {
				event.preventDefault();
				clearSearchField( event.target );
			}
		}, true );
		document.addEventListener( EVENT_TYPE_INPUT, ( event ) => {
			if ( isSearchField( event.target ) ) {
				toggleSearchClearButton( event.target );
			}
		} );
		document.addEventListener( EVENT_TYPE_KEYDOWN, ( event ) => {
			if ( isKey( event, KEY_ENTER ) ) {
				if ( isSearchClearButton( event.target ) ) {
					event.preventDefault();
					clearSearchField( event.target );

					return;
				}

				if ( isSearchField( event.target ) ) {
					const SEARCH_FIELD_CONTAINER = event.target.parentElement;
					const SEARCH_FORM = SEARCH_FIELD_CONTAINER ? SEARCH_FIELD_CONTAINER.parentElement : null;
					const SEARCH_SUBMIT = SEARCH_FORM ? getElementByClassName( CLASS_SEARCH_FORM_SUBMIT, SEARCH_FORM ) : null;

					if ( SEARCH_SUBMIT ) {
						event.preventDefault();
						SEARCH_SUBMIT.click();
					}
				}
			}
		} );
	}

	function clearSearchField( element ) {
		if ( ! isHTMLElement( element ) ) {
			return;
		}

		const SEARCH_FIELD_CONTAINER = element.parentElement;
		const SEARCH_FIELD = SEARCH_FIELD_CONTAINER ? getElementByClassName( CLASS_SEARCH_FORM_FIELD, SEARCH_FIELD_CONTAINER ) : null;

		if ( SEARCH_FIELD ) {
			removeClass( element, CLASS_SHARED_SHOW );
			SEARCH_FIELD.value = EMPTY_VALUE;
			focusElement( SEARCH_FIELD );
		}
	}

	function isSearchClearButton( element ) {
		return isHTMLElement( element, CLASS_SEARCH_FORM_CLEAR_BUTTON ) && matches( element, CLASS_SHARED_SHOW );
	}

	function isSearchField( element ) {
		return isHTMLElement( element, CLASS_SEARCH_FORM_FIELD );
	}

	function toggleSearchClearButton( element ) {
		if ( ! isHTMLElement( element, CLASS_SEARCH_FORM_FIELD ) ) {
			return;
		}

		const SEARCH_FIELD_CONTAINER = element.parentElement;
		const CLEAR_BUTTON = SEARCH_FIELD_CONTAINER ? getElementByClassName( CLASS_SEARCH_FORM_CLEAR_BUTTON, SEARCH_FIELD_CONTAINER ) : null;

		if ( CLEAR_BUTTON && element.value !== undefined ) {
			if ( element.value.length === 0 ) {
				removeClass( CLEAR_BUTTON, CLASS_SHARED_SHOW );

				return;
			}

			addClass( CLEAR_BUTTON, CLASS_SHARED_SHOW );
		}
	}

	addAccessibilityCommonEvents();
	addAnchorLinkEvents();
	addBackToTopEvents();
	addEmbeddedIframeEvents();
	addHeaderSearchEvents();
	addInputFieldEvents();
	addSearchFormEvents();
}() );
