/**
 * Theme Name: errigal
 * Theme URI: https://projecterrigal.com/themes/errigal/
 * Author: Project errigal
 * Author URI: https://projecterrigal.com/
 * Version: 1.0.4
 */
( function() {
	'use strict';

	var ATTRIBUTE_NAME_ARIA_INVALID = 'aria-invalid';
	var ATTRIBUTE_NAME_ARIA_LABEL = 'aria-label';
	var ATTRIBUTE_NAME_ARIA_HIDDEN = 'aria-hidden';
	var ATTRIBUTE_VALUE_TRUE = 'true';
	var CHARACTER_PERIOD = '.';
	var CLASS_COMPONENT_INPUT_FIELD = 'input-field';
	var CLASS_SHARED_ACTIVE = 'active';
	var CLASS_SHARED_ERROR = 'error';
	var CLASS_SHARED_ERROR_MESSAGE = 'error-message';
	var EMPTY_STRING = '';
	var EVENT_TYPE_CLICK = 'click';
	var EVENT_TYPE_SUBMIT = 'submit';
	var ID_COMMENT_FORM = 'commentform';
	var ID_COMMENT_FORM_AUTHOR = 'author';
	var ID_COMMENT_FORM_EMAIL = 'email';
	var ID_COMMENT_FORM_COMMENT = 'comment';
	var ID_COMMENT_FORM_SUBMIT = 'submit';
	var NODE_TYPE_HTML_ELEMENT = 1;
	var NODE_TYPE_DOCUMENT = 9;
	var TAG_DIV = 'div';
	var TAG_INPUT = 'input';
	var TAG_LABEL = 'label';
	var TAG_TEXTAREA = 'textarea';

	function getCommentFormAuthorField() {
		return document.getElementById( ID_COMMENT_FORM_AUTHOR );
	}

	function getCommentFormCommentField() {
		return document.getElementById( ID_COMMENT_FORM_COMMENT );
	}

	function getCommentFormEmailField() {
		return document.getElementById( ID_COMMENT_FORM_EMAIL );
	}

	function getFirstErrorCommentFormField() {
		return document.querySelector( '#'.concat( ID_COMMENT_FORM, ' .' ).concat( CLASS_SHARED_ERROR ) );
	}

	function getInputFields() {
		return document.querySelectorAll( '.'.concat( CLASS_COMPONENT_INPUT_FIELD, ' > ' ).concat( TAG_INPUT, ', .' ).concat( CLASS_COMPONENT_INPUT_FIELD, ' > ' ).concat( TAG_TEXTAREA ) );
	}

	function addClass( element, className ) {
		if ( isHTMLElement( element ) && element.classList && isString( className ) && ! isNullOrEmpty( className ) ) {
			element.classList.add( className );
		}
	}

	function matches( element, selectorString, isTag ) {
		if ( ! isHTMLElement( element ) || ! isString( selectorString ) || isNullOrEmpty( selectorString ) ) {
			return false;
		}

		if ( ! sanitiseBoolean( isTag ) && selectorString.indexOf( CHARACTER_PERIOD ) !== 0 ) {
			selectorString = CHARACTER_PERIOD + selectorString;
		}

		return element.matches ? element.matches( selectorString ) : element.msMatchesSelector( selectorString );
	}

	function removeClass( element, className ) {
		if ( isHTMLElement( element ) && element.classList && isString( className ) && ! isNullOrEmpty( className ) ) {
			element.classList.remove( className );
		}
	}

	function getElementByTagName( qualifiedName, rootElement ) {
		if ( ! isString( qualifiedName ) || isNullOrEmpty( qualifiedName ) ) {
			return null;
		}

		if ( ! rootElement ) {
			rootElement = document;
		}

		return isHTMLElement( rootElement ) ? rootElement.getElementsByTagName( qualifiedName )[ 0 ] : null;
	}

	function sanitiseBoolean( value ) {
		return isBoolean( value ) ? value : false;
	}

	function sanitiseString( value ) {
		if ( ! isString( value ) ) {
			return EMPTY_STRING;
		}

		value = value.replace( /<script[^>]*?>.*?<\/script>/gi, EMPTY_STRING );
		value = value.replace( /<\/?\w(?:[^"'>]|"[^"]*"|'[^']*')*>/g, EMPTY_STRING );
		value = value.replace( /&/g, '&amp;' );
		value = value.replace( /</g, '&lt;' );
		value = value.replace( />/g, '&gt;' );
		value = value.replace( /"/g, '&quot;' );
		value = value.replace( /'/g, '&#39;' );

		return value;
	}

	function isBoolean( value ) {
		return value === false || value === true;
	}

	function isHTMLElement( element, className ) {
		if ( ! element ) {
			return false;
		}

		var IS_HTML_ELEMENT = element.nodeType === NODE_TYPE_HTML_ELEMENT && element instanceof HTMLElement;
		var IS_DOCUMENT = element.nodeType === NODE_TYPE_DOCUMENT && element instanceof Document;
		var IS_HTML_ELEMENT_OR_DOCUMENT = IS_HTML_ELEMENT || IS_DOCUMENT;

		if ( isString( className ) && ! isNullOrEmpty( className ) ) {
			return IS_HTML_ELEMENT_OR_DOCUMENT && matches( element, className );
		}

		return IS_HTML_ELEMENT_OR_DOCUMENT;
	}

	function isNullOrEmpty( value ) {
		return value === null || value === undefined || value === EMPTY_STRING;
	}

	function isString( value ) {
		return typeof value === 'string';
	}

	function initialiseInputFieldLabelPositions() {
		var INPUT_FIELDS = getInputFields();

		for ( var i = 0; i < INPUT_FIELDS.length; i++ ) {
			if ( ! isNullOrEmpty( INPUT_FIELDS[ i ].value ) ) {
				moveInputFieldNameToLegend( INPUT_FIELDS[ i ] );
			}
		}
	}

	function getLabel( element ) {
		if ( ! isHTMLElement( element ) || ! isInputField( element ) ) {
			return null;
		}

		var INPUT_FIELD_CONTAINER = element.parentElement;

		return INPUT_FIELD_CONTAINER ? getElementByTagName( TAG_LABEL, INPUT_FIELD_CONTAINER ) : null;
	}

	function isInputField( element ) {
		if ( ! isHTMLElement( element ) || ! ( matches( element, TAG_INPUT, true ) || matches( element, TAG_TEXTAREA, true ) ) ) {
			return false;
		}

		return isHTMLElement( element.parentElement, CLASS_COMPONENT_INPUT_FIELD );
	}

	function moveInputFieldNameToLegend( element ) {
		if ( ! isHTMLElement( element ) || ! isInputField( element ) ) {
			return;
		}

		var LABEL = getLabel( element );

		if ( LABEL ) {
			addClass( LABEL, CLASS_SHARED_ACTIVE );
		}
	}

	function addCommentValidationEvents() {
		document.addEventListener( EVENT_TYPE_CLICK, function( event ) {
			if ( isSubmit( event.target ) && ! validateCommentForm() ) {
				event.preventDefault();
				focusFirstErrorCommentFormField();
			}
		} );
		document.addEventListener( EVENT_TYPE_SUBMIT, function( event ) {
			if ( isCommentForm( event.target ) ) {
				initialiseInputFieldLabelPositions();
			}
		} );
	}

	function addInputFieldError( inputField, errorMessage ) {
		if ( ! isHTMLElement( inputField ) || ! hasErrorMessage( errorMessage ) ) {
			return;
		}

		var ERROR_MESSAGE_TEXT = sanitiseString( errorMessage );

		addClass( inputField, CLASS_SHARED_ERROR );
		inputField.setAttribute( ATTRIBUTE_NAME_ARIA_LABEL, ERROR_MESSAGE_TEXT );
		inputField.setAttribute( ATTRIBUTE_NAME_ARIA_INVALID, ATTRIBUTE_VALUE_TRUE );

		var ERROR_MESSAGE = document.createElement( TAG_DIV );

		ERROR_MESSAGE.textContent = ERROR_MESSAGE_TEXT;
		addClass( ERROR_MESSAGE, CLASS_SHARED_ERROR_MESSAGE );
		ERROR_MESSAGE.setAttribute( ATTRIBUTE_NAME_ARIA_HIDDEN, ATTRIBUTE_VALUE_TRUE );
		inputField.parentNode.appendChild( ERROR_MESSAGE );
	}

	function focusFirstErrorCommentFormField() {
		var FIRST_ERROR_COMMENT_FORM_FIELD = getFirstErrorCommentFormField();

		if ( FIRST_ERROR_COMMENT_FORM_FIELD ) {
			FIRST_ERROR_COMMENT_FORM_FIELD.focus();
		}
	}

	function hasErrorMessage( errorMessage ) {
		return isString( errorMessage ) && ! isNullOrEmpty( errorMessage );
	}

	function isCommentForm( element ) {
		return isHTMLElement( element ) && element.id === ID_COMMENT_FORM;
	}

	function isSubmit( element ) {
		return isHTMLElement( element ) && element.id === ID_COMMENT_FORM_SUBMIT;
	}

	function removeInputFieldError( inputField ) {
		if ( ! isHTMLElement( inputField ) || ! isHTMLElement( inputField.parentNode ) ) {
			return;
		}

		removeClass( inputField, CLASS_SHARED_ERROR );
		inputField.removeAttribute( ATTRIBUTE_NAME_ARIA_LABEL );
		inputField.removeAttribute( ATTRIBUTE_NAME_ARIA_INVALID );

		var ERROR_MESSAGE = inputField.parentNode.querySelector( ''.concat( TAG_DIV, '.' ).concat( CLASS_SHARED_ERROR_MESSAGE ) );

		if ( ERROR_MESSAGE ) {
			ERROR_MESSAGE.parentNode.removeChild( ERROR_MESSAGE );
		}
	}

	function validateCommentForm() {
		var hasComment, hasName, hasEmail, isEmailFormatValid;

		hasComment = hasName = hasEmail = isEmailFormatValid = true;

		var COMMENT_FIELD = getCommentFormCommentField();
		var AUTHOR_FIELD = getCommentFormAuthorField();
		var EMAIL_FIELD = getCommentFormEmailField();

		if ( COMMENT_FIELD ) {
			hasComment = validateFieldFilled( COMMENT_FIELD,
				// eslint-disable-next-line no-undef
				ERRIGAL_COMMENT_FORM_ERROR.CommentRequired );
		}

		if ( AUTHOR_FIELD && AUTHOR_FIELD.required ) {
			hasName = validateFieldFilled( AUTHOR_FIELD,
				// eslint-disable-next-line no-undef
				ERRIGAL_COMMENT_FORM_ERROR.AuthorRequired );
		}

		if ( EMAIL_FIELD && EMAIL_FIELD.required ) {
			hasEmail = validateFieldFilled( EMAIL_FIELD,
				// eslint-disable-next-line no-undef
				ERRIGAL_COMMENT_FORM_ERROR.EmailAddressRequired );

			if ( hasEmail ) {
				isEmailFormatValid = validateEmailAddressFormat( EMAIL_FIELD,
					// eslint-disable-next-line no-undef
					ERRIGAL_COMMENT_FORM_ERROR.EmailAddressInvalid );
			}
		}

		return hasComment && hasName && hasEmail && isEmailFormatValid;
	}

	function validateEmailAddressFormat( inputField, errorMessage ) {
		if ( ! isHTMLElement( inputField ) || ! hasErrorMessage( errorMessage ) ) {
			return true;
		}

		removeInputFieldError( inputField );

		var REGEX_EMAIL = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;

		if ( ! REGEX_EMAIL.test( inputField.value.toLowerCase() ) ) {
			addInputFieldError( inputField, errorMessage );

			return false;
		}

		return true;
	}

	function validateFieldFilled( inputField, errorMessage ) {
		if ( ! isHTMLElement( inputField ) || ! hasErrorMessage( errorMessage ) ) {
			return true;
		}

		removeInputFieldError( inputField );

		if ( inputField.value.trim().length === 0 ) {
			addInputFieldError( inputField, errorMessage );

			return false;
		}

		return true;
	}

	addCommentValidationEvents();
}() );
