/**
 * Theme Name: errigal
 * Theme URI: https://projecterrigal.com/themes/errigal/
 * Author: Project errigal
 * Author URI: https://projecterrigal.com/
 * Version: 1.0.4
 */
( function() {
	'use strict';

	var ATTRIBUTE_NAME_HREF = 'href';
	var CHARACTER_PERIOD = '.';
	var CLASS_CONTENT_HOMEPAGE_TITLE = 'homepage-title';
	var CLASS_CUSTOMISER_PREFIX_SITE_HEADER = 'site-header-';
	var CLASS_SECTION_SITE_HEADER = 'site-header';
	var CUSTOMISER_ID_BLOGNAME = 'blogname';
	var CUSTOMISER_ID_COLOUR_SCHEME = 'colour_scheme';
	var CUSTOMISER_ID_SITE_HEADER_COLOUR = 'site_header_colour';
	var CUSTOMISER_ID_TYPOGRAPHY_BASE_FONT = 'base_font';
	var CUSTOMISER_ID_TYPOGRAPHY_FONT_SIZE = 'font_size';
	var CUSTOMISER_ID_TYPOGRAPHY_HEADING_FONT = 'heading_font';
	var CUSTOM_TYPOGRAPHY_GOOGLE_FONT_FAMILY_URL = '//fonts.googleapis.com/css?family=';
	var EMPTY_STRING = '';
	var EVENT_TYPE_CLICK_PREVIEW = 'click.preview';
	var EVENT_TYPE_PREVIEW_READY = 'preview-ready';
	var FONT_FALLBACK = '-apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, Oxygen-Sans, Ubuntu, Cantarell, "Helvetica Neue", sans-serif';
	var ID_CSS_ERRIGAL_FONT_STYLE_BASE = 'errigal-base-google-font-style-css';
	var ID_CSS_ERRIGAL_FONT_STYLE_HEADING = 'errigal-heading-google-font-style-css';
	var ID_CSS_ERRIGAL_STYLE = 'errigal-style-css';
	var ID_CSS_ERRIGAL_STYLE_INLINE = 'errigal-style-inline-css';
	var NODE_TYPE_HTML_ELEMENT = 1;
	var NODE_TYPE_DOCUMENT = 9;
	var SELECTOR_A_HREF = 'a[href]';
	var TAG_BODY = 'body';
	var TAG_HYPERLINK = 'a';

	function addClass( element, className ) {
		if ( isHTMLElement( element ) && element.classList && isString( className ) && ! isNullOrEmpty( className ) ) {
			element.classList.add( className );
		}
	}

	function matches( element, selectorString, isTag ) {
		if ( ! isHTMLElement( element ) || ! isString( selectorString ) || isNullOrEmpty( selectorString ) ) {
			return false;
		}

		if ( ! sanitiseBoolean( isTag ) && selectorString.indexOf( CHARACTER_PERIOD ) !== 0 ) {
			selectorString = CHARACTER_PERIOD + selectorString;
		}

		return element.matches ? element.matches( selectorString ) : element.msMatchesSelector( selectorString );
	}

	function removeClass( element, className ) {
		if ( isHTMLElement( element ) && element.classList && isString( className ) && ! isNullOrEmpty( className ) ) {
			element.classList.remove( className );
		}
	}

	function getElementByClassName( className, rootElement ) {
		if ( ! isString( className ) || isNullOrEmpty( className ) ) {
			return null;
		}

		if ( ! rootElement ) {
			rootElement = document;
		}

		return isHTMLElement( rootElement ) ? rootElement.getElementsByClassName( className )[ 0 ] : null;
	}

	function sanitiseBoolean( value ) {
		return isBoolean( value ) ? value : false;
	}

	function isBoolean( value ) {
		return value === false || value === true;
	}

	function isHTMLElement( element, className ) {
		if ( ! element ) {
			return false;
		}

		var IS_HTML_ELEMENT = element.nodeType === NODE_TYPE_HTML_ELEMENT && element instanceof HTMLElement;
		var IS_DOCUMENT = element.nodeType === NODE_TYPE_DOCUMENT && element instanceof Document;
		var IS_HTML_ELEMENT_OR_DOCUMENT = IS_HTML_ELEMENT || IS_DOCUMENT;

		if ( isString( className ) && ! isNullOrEmpty( className ) ) {
			return IS_HTML_ELEMENT_OR_DOCUMENT && matches( element, className );
		}

		return IS_HTML_ELEMENT_OR_DOCUMENT;
	}

	function isLinkElement( element ) {
		return Boolean( isHTMLElement( element ) && element.nodeName && element.nodeName.toLowerCase() === TAG_HYPERLINK );
	}

	function isNullOrEmpty( value ) {
		return value === null || value === undefined || value === EMPTY_STRING;
	}

	function isString( value ) {
		return typeof value === 'string';
	}

	function getCssErrigalFontStyleBase() {
		return document.getElementById( ID_CSS_ERRIGAL_FONT_STYLE_BASE );
	}

	function getCssErrigalFontStyleHeading() {
		return document.getElementById( ID_CSS_ERRIGAL_FONT_STYLE_HEADING );
	}

	function getCssErrigalStyle() {
		return document.getElementById( ID_CSS_ERRIGAL_STYLE );
	}

	function getCssErrigalStyleInline() {
		return document.getElementById( ID_CSS_ERRIGAL_STYLE_INLINE );
	}

	function getSiteHeader() {
		return getElementByClassName( CLASS_SECTION_SITE_HEADER );
	}

	function addColourSchemeEvents() {
		// eslint-disable-next-line no-undef
		wp.customize( CUSTOMISER_ID_COLOUR_SCHEME, function( value ) {
			value.bind( function( colourScheme ) {
				var CSS_ERRIGAL_STYLE = getCssErrigalStyle();

				if ( CSS_ERRIGAL_STYLE ) {
					var CSS_FILE_NAME = isNullOrEmpty( colourScheme ) ? 'style.min.css' : 'style-'.concat( colourScheme, '.min.css' );

					CSS_ERRIGAL_STYLE.setAttribute( ATTRIBUTE_NAME_HREF, CSS_ERRIGAL_STYLE.getAttribute( ATTRIBUTE_NAME_HREF ).replace( /style.*.min.css/, CSS_FILE_NAME ) );
				}
			} );
		} );
	}

	function addLinkEvents() {
		// eslint-disable-next-line no-undef
		var BODY = jQuery( TAG_BODY );

		BODY.off( EVENT_TYPE_CLICK_PREVIEW );
		BODY.on( EVENT_TYPE_CLICK_PREVIEW, SELECTOR_A_HREF, function( event ) {
			if ( isLinkToExternalSite( event.target ) ) {
				event.preventDefault();
			}
		} );
	}

	function isLinkToExternalSite( element ) {
		return isLinkElement( element ) && element.host !== window.location.host;
	}

	function addSiteHeaderColourEvents() {
		// eslint-disable-next-line no-undef
		wp.customize( CUSTOMISER_ID_SITE_HEADER_COLOUR, function( value ) {
			value.bind( function( siteHeaderColour ) {
				var SITE_HEADER = getSiteHeader();

				if ( SITE_HEADER ) {
					for ( var i = 0; i < SITE_HEADER.classList.length; i++ ) {
						var CLASS_NAME = SITE_HEADER.classList[ i ];

						if ( CLASS_NAME.includes( CLASS_CUSTOMISER_PREFIX_SITE_HEADER ) ) {
							SITE_HEADER.classList.remove( CLASS_NAME );
							SITE_HEADER.classList.add( CLASS_CUSTOMISER_PREFIX_SITE_HEADER + siteHeaderColour );

							return;
						}
					}
				}
			} );
		} );
	}

	function addStaticPageBlognameEvents() {
		// eslint-disable-next-line no-undef
		wp.customize( CUSTOMISER_ID_BLOGNAME, function( setting ) {
			setting.bind( function( blogname ) {
				var HOMEPAGE_TITLE = getElementByClassName( CLASS_CONTENT_HOMEPAGE_TITLE );

				if ( HOMEPAGE_TITLE ) {
					HOMEPAGE_TITLE.innerHTML = blogname;
				}
			} );
		} );
	}

	function addTypographyEvents() {
		var CSS_ERRIGAL_STYLE_INLINE = getCssErrigalStyleInline();

		// eslint-disable-next-line no-undef
		wp.customize( CUSTOMISER_ID_TYPOGRAPHY_BASE_FONT, function( value ) {
			value.bind( function( fontName ) {
				// eslint-disable-next-line no-undef
				var FONT_NAME = ERRIGAL_FONT_NAMES[ fontName ];
				// eslint-disable-next-line no-undef
				var FONT_FAMILY = ERRIGAL_FONT_FAMILIES[ fontName ];
				var CSS_ERRIGAL_FONT_STYLE_BASE = getCssErrigalFontStyleBase();

				if ( FONT_FAMILY && CSS_ERRIGAL_FONT_STYLE_BASE && CSS_ERRIGAL_STYLE_INLINE ) {
					CSS_ERRIGAL_FONT_STYLE_BASE.setAttribute( ATTRIBUTE_NAME_HREF, ''.concat( CUSTOM_TYPOGRAPHY_GOOGLE_FONT_FAMILY_URL ).concat( FONT_FAMILY ) );
					CSS_ERRIGAL_STYLE_INLINE.sheet.cssRules[ 0 ].style.fontFamily = '"'.concat( FONT_NAME, '", ' ).concat( FONT_FALLBACK );
				}
			} );
		} );

		// eslint-disable-next-line no-undef
		wp.customize( CUSTOMISER_ID_TYPOGRAPHY_HEADING_FONT, function( value ) {
			value.bind( function( fontName ) {
				// eslint-disable-next-line no-undef
				var FONT_NAME = ERRIGAL_FONT_NAMES[ fontName ];
				// eslint-disable-next-line no-undef
				var FONT_FAMILY = ERRIGAL_FONT_FAMILIES[ fontName ];
				var CSS_ERRIGAL_FONT_STYLE_HEADING = getCssErrigalFontStyleHeading();

				if ( FONT_FAMILY && CSS_ERRIGAL_FONT_STYLE_HEADING && CSS_ERRIGAL_STYLE_INLINE ) {
					CSS_ERRIGAL_FONT_STYLE_HEADING.setAttribute( ATTRIBUTE_NAME_HREF, ''.concat( CUSTOM_TYPOGRAPHY_GOOGLE_FONT_FAMILY_URL ).concat( FONT_FAMILY ) );
					CSS_ERRIGAL_STYLE_INLINE.sheet.cssRules[ 1 ].style.fontFamily = '"'.concat( FONT_NAME, '", ' ).concat( FONT_FALLBACK );
				}
			} );
		} );

		// eslint-disable-next-line no-undef
		wp.customize( CUSTOMISER_ID_TYPOGRAPHY_FONT_SIZE, function( value ) {
			value.bind( function( fontSize ) {
				// eslint-disable-next-line no-undef
				var FONT_SIZES = ERRIGAL_FONT_SIZES;

				if ( fontSize && FONT_SIZES && FONT_SIZES.includes( fontSize ) ) {
					for ( var i = 0; i < FONT_SIZES.length; i++ ) {
						removeClass( document.documentElement, FONT_SIZES[ i ] );
					}

					addClass( document.documentElement, fontSize );
				}
			} );
		} );
	}

	// eslint-disable-next-line no-undef
	wp.customize.bind( EVENT_TYPE_PREVIEW_READY, function() {
		addColourSchemeEvents();
		addLinkEvents();
		addSiteHeaderColourEvents();
		addStaticPageBlognameEvents();
		addTypographyEvents();
	} );
}() );
