/**
 * Theme Name: errigal
 * Theme URI: https://projecterrigal.com/themes/errigal/
 * Author: Project errigal
 * Author URI: https://projecterrigal.com/
 * Version: 1.0.4
 */
( function() {
	'use strict';

	var ATTRIBUTE_NAME_ARIA_EXPANDED = 'aria-expanded';
	var ATTRIBUTE_NAME_ARIA_HIDDEN = 'aria-hidden';
	var ATTRIBUTE_NAME_STYLE = 'style';
	var ATTRIBUTE_VALUE_FALSE = 'false';
	var ATTRIBUTE_VALUE_TRUE = 'true';
	var CALLBACK_INITIAL_ATTEMPT_COUNT = 0;
	var CALLBACK_INITIAL_INTERVAL = 0;
	var CALLBACK_INTERVAL = 50;
	var CALLBACK_MAX_ATTEMPT_COUNT = 20;
	var CHARACTER_HASH = '#';
	var CHARACTER_PERIOD = '.';
	var CLASS_COMPONENT_GRID_ROW = 'row';
	var CLASS_NAV_CLOSE_MENU_BUTTON_CONTAINER = 'close-menu-button__container';
	var CLASS_NAV_HEADER_MENU_BUTTON = 'site-header__menu-button';
	var CLASS_NAV_MAIN_NAVIGATION = 'main-navigation';
	var CLASS_NAV_MENU = 'menu';
	var CLASS_NAV_MENU_CONTAINER = 'menu-container';
	var CLASS_NAV_MENU_ITEM = 'menu-item';
	var CLASS_NAV_MENU_WITH_SUB_MENU = 'menu-item-has-children';
	var CLASS_NAV_MOBILE_MENU_BACKGROUND = 'main-navigation-background';
	var CLASS_NAV_MOBILE_MENU_BUTTON = 'mobile-navigation__menu-button';
	var CLASS_NAV_MOBILE_MENU_CLOSE_BUTTON = 'close-menu-button';
	var CLASS_NAV_SUB_MENU = 'sub-menu';
	var CLASS_NAV_SUB_MENU_TOGGLE = 'sub-menu-toggle';
	var CLASS_NAV_SUB_MENU_TOGGLE_TEXT_CLOSE = 'sub-menu-toggle-text-close';
	var CLASS_NAV_SUB_MENU_TOGGLE_TEXT_OPEN = 'sub-menu-toggle-text-open';
	var CLASS_SECTION_CONTAINER = 'container';
	var CLASS_SECTION_SITE_CONTENT = 'site-content';
	var CLASS_SECTION_SITE_HEADER = 'site-header';
	var CLASS_SHARED_MENU_FLIP = 'menu-flip';
	var CLASS_SHARED_MOBILE_DEVICE = 'mobile-device';
	var CLASS_SHARED_NO_SCROLL = 'no-scroll';
	var CLASS_SHARED_SHOW = 'show';
	var CLASS_SHARED_STICKY = 'sticky';
	var IS_TO_OPEN_SUB_MENU = true;
	var IS_TO_CLOSE_SUB_MENU = false;
	var DELAY_SHORT = 100;
	var DELAY_SHORTER = 50;
	var EMPTY_STRING = '';
	var EVENT_TYPE_CLICK = 'click';
	var EVENT_TYPE_FOCUSIN = 'focusin';
	var EVENT_TYPE_KEYDOWN = 'keydown';
	var EVENT_TYPE_LOAD = 'load';
	var EVENT_TYPE_MOUSEDOWN = 'mousedown';
	var EVENT_TYPE_MOUSEUP = 'mouseup';
	var EVENT_TYPE_RESIZE = 'resize';
	var EVENT_TYPE_SCROLL = 'scroll';
	var KEY_DOWN = [ 'ArrowDown', '40' ];
	var KEY_ENTER = [ 'Enter', '13' ];
	var KEY_ESCAPE = [ 'Escape', '27' ];
	var KEY_TAB = [ 'Tab', '9' ];
	var KEY_UP = [ 'ArrowUp', '38' ];
	var KEY_ID_KEY = 0;
	var KEY_ID_KEYCODE = 1;
	var NODE_TYPE_HTML_ELEMENT = 1;
	var NODE_TYPE_DOCUMENT = 9;
	var POSITION_FIXED = 'fixed';
	var TAG_HYPERLINK = 'a';

	function focusElementCallback( element, interval, attemptCount ) {
		if ( ! isHTMLElement( element ) || isNaN( interval ) || isNaN( attemptCount ) || attemptCount === CALLBACK_MAX_ATTEMPT_COUNT ) {
			return;
		}

		setTimeout( function() {
			element.focus();

			if ( element === document.activeElement ) {
				return;
			}

			attemptCount++;
			focusElementCallback( element, CALLBACK_INTERVAL, attemptCount );
		}, interval );
	}

	function addClass( element, className ) {
		if ( isHTMLElement( element ) && element.classList && isString( className ) && ! isNullOrEmpty( className ) ) {
			element.classList.add( className );
		}
	}

	function matches( element, selectorString, isTag ) {
		if ( ! isHTMLElement( element ) || ! isString( selectorString ) || isNullOrEmpty( selectorString ) ) {
			return false;
		}

		if ( ! sanitiseBoolean( isTag ) && selectorString.indexOf( CHARACTER_PERIOD ) !== 0 ) {
			selectorString = CHARACTER_PERIOD + selectorString;
		}

		return element.matches ? element.matches( selectorString ) : element.msMatchesSelector( selectorString );
	}

	function removeClass( element, className ) {
		if ( isHTMLElement( element ) && element.classList && isString( className ) && ! isNullOrEmpty( className ) ) {
			element.classList.remove( className );
		}
	}

	function removeStyle( element ) {
		if ( isHTMLElement( element ) ) {
			element.removeAttribute( ATTRIBUTE_NAME_STYLE );
		}
	}

	function getElementByClassName( className, rootElement ) {
		if ( ! isString( className ) || isNullOrEmpty( className ) ) {
			return null;
		}

		if ( ! rootElement ) {
			rootElement = document;
		}

		return isHTMLElement( rootElement ) ? rootElement.getElementsByClassName( className )[ 0 ] : null;
	}

	function getElementByTagName( qualifiedName, rootElement ) {
		if ( ! isString( qualifiedName ) || isNullOrEmpty( qualifiedName ) ) {
			return null;
		}

		if ( ! rootElement ) {
			rootElement = document;
		}

		return isHTMLElement( rootElement ) ? rootElement.getElementsByTagName( qualifiedName )[ 0 ] : null;
	}

	function focusElement( element ) {
		if ( ! isHTMLElement( element ) ) {
			return;
		}

		var attemptCount = CALLBACK_INITIAL_ATTEMPT_COUNT;

		focusElementCallback( element, CALLBACK_INITIAL_INTERVAL, attemptCount );
	}

	function isDescendantOf( element, parentElement ) {
		if ( ! isHTMLElement( element ) || ! isHTMLElement( parentElement ) ) {
			return false;
		}

		var target = element;

		while ( target.parentElement ) {
			if ( parentElement === target.parentElement ) {
				return true;
			}

			target = target.parentElement;
		}

		return false;
	}

	function isFixed( element ) {
		if ( ! isHTMLElement( element ) ) {
			return false;
		}

		return window.getComputedStyle( element, null ).position.toLowerCase() === POSITION_FIXED;
	}

	function isMobile() {
		return matches( document.body, CLASS_SHARED_MOBILE_DEVICE );
	}

	function isKey( event, key ) {
		if ( ! isEvent( event ) || ! isStringPair( key ) ) {
			return false;
		}

		if ( isIe() ) {
			return event.keyCode === parseInt( key[ KEY_ID_KEYCODE ] );
		}

		return event.key === key[ KEY_ID_KEY ];
	}

	function isIe() {
		return /(trident|msie)/i.test( navigator.userAgent );
	}

	function sanitiseBoolean( value ) {
		return isBoolean( value ) ? value : false;
	}

	function isBoolean( value ) {
		return value === false || value === true;
	}

	function isEvent( event ) {
		return /\[object (?:\w+)?Event\]/.test( Object.prototype.toString.call( event ) ) && 'type' in event && 'target' in event;
	}

	function isHTMLElement( element, className ) {
		if ( ! element ) {
			return false;
		}

		var IS_HTML_ELEMENT = element.nodeType === NODE_TYPE_HTML_ELEMENT && element instanceof HTMLElement;
		var IS_DOCUMENT = element.nodeType === NODE_TYPE_DOCUMENT && element instanceof Document;
		var IS_HTML_ELEMENT_OR_DOCUMENT = IS_HTML_ELEMENT || IS_DOCUMENT;

		if ( isString( className ) && ! isNullOrEmpty( className ) ) {
			return IS_HTML_ELEMENT_OR_DOCUMENT && matches( element, className );
		}

		return IS_HTML_ELEMENT_OR_DOCUMENT;
	}

	function isLinkElement( element ) {
		return Boolean( isHTMLElement( element ) && element.nodeName && element.nodeName.toLowerCase() === TAG_HYPERLINK );
	}

	function isNullOrEmpty( value ) {
		return value === null || value === undefined || value === EMPTY_STRING;
	}

	function isString( value ) {
		return typeof value === 'string';
	}

	function isStringPair( value ) {
		if ( ! Array.isArray( value ) || value.length !== 2 ) {
			return false;
		}

		for ( var i = 0; i < value.length; i++ ) {
			if ( ! isString( value[ i ] ) || value[ i ].length === 0 ) {
				return false;
			}
		}

		return true;
	}

	function getHeaderMenuButton() {
		return getElementByClassName( CLASS_NAV_HEADER_MENU_BUTTON );
	}

	function getLastTopLevelMobileMenuItemLink() {
		return document.querySelector( '.'.concat( CLASS_NAV_MENU_CONTAINER, '.' ).concat( CLASS_SHARED_SHOW, ' .' ).concat( CLASS_NAV_MENU, ':last-of-type > .' ).concat( CLASS_NAV_MENU_ITEM, ':last-of-type > ' ).concat( TAG_HYPERLINK ) );
	}

	function getMainNavigation() {
		return getElementByClassName( CLASS_NAV_MAIN_NAVIGATION );
	}

	function getMenu() {
		var MENU_CONTAINER = getMenuContainer();

		return MENU_CONTAINER ? getElementByClassName( CLASS_NAV_MENU, MENU_CONTAINER ) : null;
	}

	function getMenuBackground() {
		return getElementByClassName( CLASS_NAV_MOBILE_MENU_BACKGROUND );
	}

	function getMenuContainer() {
		return getElementByClassName( CLASS_NAV_MENU_CONTAINER );
	}

	function getMobileMenuButton() {
		return getElementByClassName( CLASS_NAV_MOBILE_MENU_BUTTON );
	}

	function getMobileMenuCloseButton() {
		return getElementByClassName( CLASS_NAV_MOBILE_MENU_CLOSE_BUTTON );
	}

	function getOpenedTopLevelDesktopMenuItemLink() {
		var TOP_LEVEL_DESKTOP_MENU_ITEMS = document.querySelectorAll( '.'.concat( CLASS_NAV_MENU_CONTAINER, ':not(.' ).concat( CLASS_SHARED_SHOW, ') .' ).concat( CLASS_NAV_MENU, ' > .' ).concat( CLASS_NAV_MENU_ITEM ) );

		for ( var i = 0; i < TOP_LEVEL_DESKTOP_MENU_ITEMS.length; i++ ) {
			if ( matches( TOP_LEVEL_DESKTOP_MENU_ITEMS[ i ], CLASS_SHARED_SHOW ) ) {
				return getElementByTagName( TAG_HYPERLINK, TOP_LEVEL_DESKTOP_MENU_ITEMS[ i ] );
			}
		}

		return null;
	}

	function getSiteContentContainer() {
		return document.querySelector( '.'.concat( CLASS_SECTION_SITE_CONTENT, ' > .' ).concat( CLASS_SECTION_CONTAINER ) );
	}

	function getSiteHeader() {
		return getElementByClassName( CLASS_SECTION_SITE_HEADER );
	}

	var mouseDown = false;

	function addMenuEvents() {
		document.addEventListener( EVENT_TYPE_CLICK, function( event ) {
			if ( isInMainNavigation( event.target ) ) {
				if ( isSubMenuToggle( event.target ) || isLinkOfMenuItemWithSubMenuWithNoHref( event.target ) ) {
					event.preventDefault();

					if ( isOpenMenuItem( event.target ) ) {
						closeAllMenusAtSameLevelAndBelow( event.target );
						focusElement( event.target );

						return;
					}

					closeAllMenusAtSameLevelAndBelow( event.target );
					openSubMenu( event, event.target );
					focusFirstSubMenuItemLink( event.target );

					if ( ! isInOpenMobileMenu( event.target ) ) {
						setSubMenuDirection( event.target );
					}

					return;
				}

				if ( isMenuItemLink( event.target ) ) {
					event.stopPropagation();

					return;
				}
			}

			if ( isHeaderMenuButton( event.target ) || isMobileMenuButton( event.target ) ) {
				openMobileMenu();
				focusMobileMenuCloseButton();

				return;
			}

			if ( isMobileMenuBackground( event.target ) || isMobileMenuCloseButton( event.target ) ) {
				closeMobileMenu();

				if ( isMobile() ) {
					focusMobileMenuButton();

					return;
				}

				focusHeaderMenuButton();

				return;
			}

			if ( isOpenMobileMenu( event.target ) || isOpenMobileMenuContainer( event.target ) || isCloseMenuButtonContainer( event.target ) ) {
				event.preventDefault();

				return;
			}

			closeMenu();
		} );
		document.addEventListener( EVENT_TYPE_FOCUSIN, function( event ) {
			if ( ( isMenuItemLink( event.target ) || isSubMenuToggleOpened( event.target ) ) && ! mouseDown ) {
				closeAllMenusAtSameLevelAndBelow( event.target );
			}
		} );
		document.addEventListener( EVENT_TYPE_KEYDOWN, function( event ) {
			if ( isKey( event, KEY_DOWN ) ) {
				if ( isInMainNavigation( event.target ) ) {
					if ( isLinkOfNotLastSubMenuItem( event.target ) || isLinkOfNotLastTopLevelMobileMenuItem( event.target ) ) {
						event.preventDefault();
						focusNextMenuItemLink( event.target );

						return;
					}

					if ( isLastSubMenuItemLink( event.target ) ) {
						event.preventDefault();

						return;
					}
				}
			}

			if ( isKey( event, KEY_ENTER ) ) {
				if ( isHeaderMenuButton( event.target ) || isMobileMenuButton( event.target ) ) {
					event.preventDefault();
					openMobileMenu();
					focusMobileMenuCloseButton();

					return;
				}

				if ( isMobileMenuCloseButton( event.target ) ) {
					event.preventDefault();
					closeMobileMenu();

					if ( isMobile() ) {
						focusMobileMenuButton();

						return;
					}

					focusHeaderMenuButton();

					return;
				}

				if ( isInMainNavigation( event.target ) && ( isSubMenuToggleClosed( event.target ) || isLinkOfMenuItemWithSubMenuWithNoHref( event.target ) ) ) {
					event.preventDefault();
					openSubMenu( event, event.target );
					focusFirstSubMenuItemLink( event.target );

					if ( isSubMenuToggleOrTopLevelMenuItemWithNoHref( event.target ) && ! isInOpenMobileMenu( event.target ) ) {
						setSubMenuDirection( event.target );
					}

					return;
				}
			}

			if ( isKey( event, KEY_ESCAPE ) ) {
				if ( mobileMenuIsOpen() ) {
					event.preventDefault();
					closeMobileMenu();

					if ( isMobile() ) {
						focusMobileMenuButton();

						return;
					}

					focusHeaderMenuButton();

					return;
				}

				if ( ! isMobile() ) {
					var OPENED_TOP_LEVEL_DESKTOP_MENU_ITEM_LINK = getOpenedTopLevelDesktopMenuItemLink();

					if ( OPENED_TOP_LEVEL_DESKTOP_MENU_ITEM_LINK ) {
						event.preventDefault();
						closeMenu();
						focusElement( OPENED_TOP_LEVEL_DESKTOP_MENU_ITEM_LINK );

						return;
					}
				}
			}

			if ( isKey( event, KEY_TAB ) ) {
				if ( isInMainNavigation( event.target ) ) {
					if ( event.shiftKey && isMobileMenuCloseButton( event.target ) ) {
						focusLastTopLevelMobileMenuItemLink();

						return;
					}

					if ( ! event.shiftKey && isEndOfMenu( event.target ) ) {
						closeMenu();

						if ( isInOpenMobileMenu( event.target ) && isEndOfMainNavigation( event.target ) ) {
							focusMobileMenuCloseButton();
						}

						return;
					}
				}
			}

			if ( isKey( event, KEY_UP ) ) {
				if ( isInMainNavigation( event.target ) ) {
					if ( isLinkOfNotFirstSubMenuItem( event.target ) || isLinkOfNotFirstTopLevelMobileMenuItem( event.target ) ) {
						event.preventDefault();
						focusPreviousMenuItemLink( event.target );

						return;
					}

					if ( isMenuItemLink( event.target ) ) {
						event.preventDefault();
					}
				}
			}
		} );
		document.addEventListener( EVENT_TYPE_MOUSEDOWN, function() {
			mouseDown = true;
		} );
		document.addEventListener( EVENT_TYPE_MOUSEUP, function() {
			mouseDown = false;
		} );
		window.addEventListener( EVENT_TYPE_RESIZE, function() {
			if ( ! isMobile() ) {
				if ( mobileMenuIsOpen() ) {
					closeMobileMenu();

					return;
				}

				closeMenu();
			}
		} );
		window.addEventListener( EVENT_TYPE_SCROLL, function() {
			closeMenu();
		} );
	}

	function isInMainNavigation( element ) {
		if ( ! isHTMLElement( element ) ) {
			return false;
		}

		var MAIN_NAVIGATION = getMainNavigation();

		return isDescendantOf( element, MAIN_NAVIGATION );
	}

	function isLinkOfMenuItemWithSubMenu( element ) {
		if ( ! isLinkElement( element ) ) {
			return false;
		}

		return isHTMLElement( element.parentElement, CLASS_NAV_MENU_WITH_SUB_MENU );
	}

	function closeAllMenusAtSameLevelAndBelow( element ) {
		if ( ! isHTMLElement( element ) ) {
			return;
		}

		var MENU_ITEM = element.parentElement;
		var PARENT_MENU_ITEM = MENU_ITEM ? MENU_ITEM.parentElement : null;

		if ( PARENT_MENU_ITEM ) {
			if ( matches( PARENT_MENU_ITEM, CLASS_NAV_MENU ) ) {
				closeMenu();
			}

			closeDescendantMenuItems( PARENT_MENU_ITEM );
		}
	}

	function closeDescendantMenuItems( element ) {
		if ( ! isHTMLElement( element ) ) {
			return;
		}

		var OPEN_MENUS = element.querySelectorAll( '.'.concat( CLASS_SHARED_SHOW ) );

		for ( var i = 0; i < OPEN_MENUS.length; i++ ) {
			removeClass( OPEN_MENUS[ i ], CLASS_SHARED_SHOW );
			removeClass( OPEN_MENUS[ i ], CLASS_SHARED_MENU_FLIP );
			toggleSubMenuToggle( OPEN_MENUS[ i ], IS_TO_CLOSE_SUB_MENU );
		}
	}

	function closeMenu() {
		var MENU_CONTAINER = getMenuContainer();

		if ( MENU_CONTAINER ) {
			closeDescendantMenuItems( MENU_CONTAINER );
		}
	}

	function closeMobileMenu() {
		var MENU_BACKGROUND = getMenuBackground();
		var MENU_CONTAINER = getMenuContainer();

		if ( MENU_BACKGROUND && MENU_CONTAINER ) {
			closeMenu();
			removeClass( MENU_CONTAINER, CLASS_SHARED_SHOW );
			setTimeout( function() {
				removeClass( MENU_BACKGROUND, CLASS_SHARED_SHOW );
				removeClass( document.body, CLASS_SHARED_NO_SCROLL );
			}, DELAY_SHORT );
		}
	}

	function focusFirstSubMenuItemLink( element ) {
		if ( ! isHTMLElement( element ) || ! isHTMLElement( element.parentElement ) ) {
			return;
		}

		var MENU_ITEM = element.parentElement;
		var SUB_MENU = MENU_ITEM ? getElementByClassName( CLASS_NAV_SUB_MENU, MENU_ITEM ) : null;
		var FIRST_SUB_MENU_ITEM = SUB_MENU ? SUB_MENU.firstElementChild : null;
		var FIRST_SUB_MENU_ITEM_LINK = FIRST_SUB_MENU_ITEM && matches( FIRST_SUB_MENU_ITEM, CLASS_NAV_MENU_ITEM ) ? getElementByTagName( TAG_HYPERLINK, FIRST_SUB_MENU_ITEM ) : null;

		if ( FIRST_SUB_MENU_ITEM_LINK ) {
			focusElement( FIRST_SUB_MENU_ITEM_LINK );
		}
	}

	function focusHeaderMenuButton() {
		var HEADER_MENU_BUTTON = getHeaderMenuButton();

		if ( HEADER_MENU_BUTTON ) {
			focusElement( HEADER_MENU_BUTTON );
		}
	}

	function focusLastTopLevelMobileMenuItemLink() {
		var LAST_TOP_LEVEL_MOBILE_MENU_ITEM_LINK = getLastTopLevelMobileMenuItemLink();

		if ( LAST_TOP_LEVEL_MOBILE_MENU_ITEM_LINK ) {
			focusElement( LAST_TOP_LEVEL_MOBILE_MENU_ITEM_LINK );
		}
	}

	function focusMobileMenuButton() {
		var MOBILE_MENU_BUTTON = getMobileMenuButton();

		if ( MOBILE_MENU_BUTTON ) {
			focusElement( MOBILE_MENU_BUTTON );
		}
	}

	function focusMobileMenuCloseButton() {
		var MOBILE_MENU_CLOSE_BUTTON = getMobileMenuCloseButton();

		if ( MOBILE_MENU_CLOSE_BUTTON ) {
			focusElement( MOBILE_MENU_CLOSE_BUTTON );
		}
	}

	function focusNextMenuItemLink( element ) {
		if ( ! isHTMLElement( element ) ) {
			return;
		}

		var NEXT_MENU_ITEM = getNextMenuItem( element );
		var NEXT_MENU_ITEM_LINK = NEXT_MENU_ITEM ? getElementByTagName( TAG_HYPERLINK, NEXT_MENU_ITEM ) : null;

		if ( NEXT_MENU_ITEM_LINK ) {
			NEXT_MENU_ITEM_LINK.focus();
		}
	}

	function focusPreviousMenuItemLink( element ) {
		if ( ! isHTMLElement( element ) ) {
			return;
		}

		var PREVIOUS_MENU_ITEM = getPreviousMenuItem( element );
		var PREVIOUS_MENU_ITEM_LINK = PREVIOUS_MENU_ITEM ? getElementByTagName( TAG_HYPERLINK, PREVIOUS_MENU_ITEM ) : null;

		if ( PREVIOUS_MENU_ITEM_LINK ) {
			PREVIOUS_MENU_ITEM_LINK.focus();
		}
	}

	function getNextMenuItem( element ) {
		if ( ! isHTMLElement( element ) ) {
			return false;
		}

		var MENU_ITEM = element.parentElement;

		return isHTMLElement( MENU_ITEM, CLASS_NAV_MENU_ITEM ) && isHTMLElement( MENU_ITEM.nextElementSibling, CLASS_NAV_MENU_ITEM ) ? MENU_ITEM.nextElementSibling : null;
	}

	function getPreviousMenuItem( element ) {
		if ( ! isHTMLElement( element ) ) {
			return null;
		}

		var MENU_ITEM = element.parentElement;

		return isHTMLElement( MENU_ITEM, CLASS_NAV_MENU_ITEM ) && isHTMLElement( MENU_ITEM.previousElementSibling, CLASS_NAV_MENU_ITEM ) ? MENU_ITEM.previousElementSibling : null;
	}

	function isCloseMenuButtonContainer( element ) {
		return isHTMLElement( element, CLASS_NAV_CLOSE_MENU_BUTTON_CONTAINER );
	}

	function isEndOfMainNavigation( element ) {
		if ( ! isHTMLElement( element ) ) {
			return false;
		}

		var MAIN_NAVIGATION = getElementByClassName( CLASS_NAV_MAIN_NAVIGATION );
		var MENUS = MAIN_NAVIGATION ? MAIN_NAVIGATION.getElementsByClassName( CLASS_NAV_MENU ) : null;
		var LAST_MENU = MENUS && MENUS.length !== 0 ? MENUS[ MENUS.length - 1 ] : null;

		return isDescendantOf( element, LAST_MENU );
	}

	function isEndOfMenu( element ) {
		if ( ! isHTMLElement( element ) || ! ( isMenuItemLink( element ) || isSubMenuToggleClosed( element ) ) || isLinkOfMenuItemWithSubMenu( element ) ) {
			return false;
		}

		return isEndOfMenuCallback( element.parentElement );
	}

	function isEndOfMenuCallback( element ) {
		if ( ! isHTMLElement( element, CLASS_NAV_MENU_ITEM ) || ! element.parentElement || element.nextElementSibling ) {
			return false;
		}

		if ( isHTMLElement( element.parentElement, CLASS_NAV_SUB_MENU ) ) {
			return isEndOfMenuCallback( element.parentElement.parentElement );
		}

		return isHTMLElement( element.parentElement, CLASS_NAV_MENU );
	}

	function isHeaderMenuButton( element ) {
		return isHTMLElement( element, CLASS_NAV_HEADER_MENU_BUTTON );
	}

	function isInOpenMobileMenu( element ) {
		if ( ! isHTMLElement( element ) ) {
			return false;
		}

		var MENU_CONTAINER = getMenuContainer();

		return isHTMLElement( MENU_CONTAINER, CLASS_SHARED_SHOW ) && isDescendantOf( element, MENU_CONTAINER );
	}

	function isLastSubMenuItemLink( element ) {
		return isSubMenuItemLink( element ) && ! getNextMenuItem( element );
	}

	function isLinkElementWithNoHref( element ) {
		return isLinkElement( element ) && Boolean( ! element.href || element.href.charAt( element.href.length - 1 ) === CHARACTER_HASH );
	}

	function isLinkOfMenuItemWithSubMenuWithNoHref( element ) {
		return isLinkOfMenuItemWithSubMenu( element ) && isLinkElementWithNoHref( element );
	}

	function isLinkOfNotFirstSubMenuItem( element ) {
		if ( ! isSubMenuItemLink( element ) ) {
			return false;
		}

		var PREVIOUS_MENU_ITEM = getPreviousMenuItem( element );

		return isMenuItem( PREVIOUS_MENU_ITEM );
	}

	function isLinkOfNotFirstTopLevelMobileMenuItem( element ) {
		if ( ! isTopLevelMenuItemLink( element ) ) {
			return false;
		}

		var PREVIOUS_MENU_ITEM = getPreviousMenuItem( element );

		return isMenuItem( PREVIOUS_MENU_ITEM ) && isInOpenMobileMenu( element );
	}

	function isLinkOfNotLastSubMenuItem( element ) {
		if ( ! isSubMenuItemLink( element ) ) {
			return false;
		}

		var NEXT_MENU_ITEM = getNextMenuItem( element );

		return isMenuItem( NEXT_MENU_ITEM );
	}

	function isLinkOfNotLastTopLevelMobileMenuItem( element ) {
		if ( ! isTopLevelMenuItemLink( element ) ) {
			return false;
		}

		var NEXT_MENU_ITEM = getNextMenuItem( element );

		return isMenuItem( NEXT_MENU_ITEM ) && isInOpenMobileMenu( element );
	}

	function isMenuItem( element ) {
		return isHTMLElement( element, CLASS_NAV_MENU_ITEM );
	}

	function isMenuItemLink( element ) {
		if ( ! isLinkElement( element ) ) {
			return false;
		}

		return isMenuItem( element.parentElement );
	}

	function isMobileMenuBackground( element ) {
		return isHTMLElement( element, CLASS_NAV_MOBILE_MENU_BACKGROUND );
	}

	function isMobileMenuButton( element ) {
		return isHTMLElement( element, CLASS_NAV_MOBILE_MENU_BUTTON );
	}

	function isMobileMenuCloseButton( element ) {
		return isHTMLElement( element, CLASS_NAV_MOBILE_MENU_CLOSE_BUTTON );
	}

	function isOpenMenuItem( element ) {
		if ( ! isHTMLElement( element ) ) {
			return false;
		}

		var MENU_ITEM = element.parentElement;

		return isHTMLElement( MENU_ITEM, CLASS_NAV_MENU_ITEM ) && matches( MENU_ITEM, CLASS_SHARED_SHOW );
	}

	function isOpenMobileMenu( element ) {
		if ( ! isHTMLElement( element, CLASS_COMPONENT_GRID_ROW ) ) {
			return false;
		}

		var MENU_CONTAINER = element.parentElement;

		return isHTMLElement( MENU_CONTAINER, CLASS_NAV_MENU_CONTAINER ) && matches( MENU_CONTAINER, CLASS_SHARED_SHOW );
	}

	function isOpenMobileMenuContainer( element ) {
		return isHTMLElement( element, CLASS_NAV_MENU_CONTAINER ) && matches( element, CLASS_SHARED_SHOW );
	}

	function isSubMenuItemLink( element ) {
		if ( ! isLinkElement( element ) || ! isMenuItemLink( element ) ) {
			return false;
		}

		var MENU_ITEM = element.parentElement;
		var PARENT_SUB_MENU = MENU_ITEM ? MENU_ITEM.parentElement : null;

		return isHTMLElement( PARENT_SUB_MENU, CLASS_NAV_SUB_MENU );
	}

	function isSubMenuToggle( element ) {
		return isHTMLElement( element, CLASS_NAV_SUB_MENU_TOGGLE );
	}

	function isSubMenuToggleClosed( element ) {
		if ( ! isSubMenuToggle( element ) ) {
			return false;
		}

		var MENU_ITEM = element.parentElement;

		return isMenuItem( MENU_ITEM ) && ! matches( MENU_ITEM, CLASS_SHARED_SHOW );
	}

	function isSubMenuToggleOpened( element ) {
		if ( ! isSubMenuToggle( element ) ) {
			return false;
		}

		var MENU_ITEM = element.parentElement;

		return isMenuItem( MENU_ITEM ) && matches( MENU_ITEM, CLASS_SHARED_SHOW );
	}

	function isSubMenuToggleOrTopLevelMenuItemWithNoHref( element ) {
		return isSubMenuToggle( element ) || isTopLevelMenuItemLinkWithNoHref( element );
	}

	function isTopLevelMenuItemLink( element ) {
		if ( ! isMenuItemLink( element ) ) {
			return false;
		}

		var MENU_ITEM = element.parentElement;
		var MENU = isMenuItem( MENU_ITEM ) ? MENU_ITEM.parentElement : null;

		return isHTMLElement( MENU, CLASS_NAV_MENU );
	}

	function isTopLevelMenuItemLinkWithNoHref( element ) {
		return isTopLevelMenuItemLink( element ) && isLinkElementWithNoHref( element );
	}

	function mobileMenuIsOpen() {
		var MENU_CONTAINER = getMenuContainer();

		return isHTMLElement( MENU_CONTAINER, CLASS_SHARED_SHOW );
	}

	function openMobileMenu() {
		var MENU_BACKGROUND = getMenuBackground();
		var MENU_CONTAINER = getMenuContainer();
		var MENU = getMenu();
		var CLOSE_BUTTON = getMobileMenuCloseButton();

		if ( MENU_BACKGROUND && MENU_CONTAINER && MENU && CLOSE_BUTTON ) {
			addClass( document.body, CLASS_SHARED_NO_SCROLL );
			addClass( MENU_BACKGROUND, CLASS_SHARED_SHOW );
			setTimeout( function() {
				addClass( MENU_CONTAINER, CLASS_SHARED_SHOW );
				MENU.scrollIntoView();
			}, DELAY_SHORTER );
		}
	}

	function openSubMenu( event, element ) {
		if ( ! isEvent( event ) || ! isHTMLElement( element ) ) {
			return;
		}

		var MENU_ITEM = element.parentElement;
		var MENU_ITEM_LINK = element;
		var SUB_MENU = MENU_ITEM ? getElementByClassName( CLASS_NAV_SUB_MENU, MENU_ITEM ) : null;

		if ( MENU_ITEM && MENU_ITEM_LINK && SUB_MENU ) {
			event.preventDefault();
			event.stopPropagation();
			addClass( MENU_ITEM, CLASS_SHARED_SHOW );
			addClass( SUB_MENU, CLASS_SHARED_SHOW );
			toggleSubMenuToggle( MENU_ITEM, IS_TO_OPEN_SUB_MENU );
		}
	}

	function setSubMenuDirection( element ) {
		if ( ! isHTMLElement( element ) ) {
			return;
		}

		var MENU_ITEM = element.parentElement;
		var SUB_MENU = MENU_ITEM ? getElementByClassName( CLASS_NAV_SUB_MENU, MENU_ITEM ) : null;

		if ( isHTMLElement( SUB_MENU, CLASS_NAV_SUB_MENU ) ) {
			if ( matches( SUB_MENU, CLASS_SHARED_SHOW ) && document.body.clientWidth < SUB_MENU.getBoundingClientRect().left + SUB_MENU.offsetWidth ) {
				addClass( SUB_MENU, CLASS_SHARED_MENU_FLIP );

				return;
			}

			removeClass( SUB_MENU, CLASS_SHARED_MENU_FLIP );
		}
	}

	function toggleSubMenuToggle( element, isToOpenSubMenu ) {
		if ( ! isHTMLElement( element, CLASS_NAV_MENU_WITH_SUB_MENU ) || ! isBoolean( isToOpenSubMenu ) ) {
			return;
		}

		var SUB_MENU_TOGGLE = getElementByClassName( CLASS_NAV_SUB_MENU_TOGGLE, element );

		if ( ! SUB_MENU_TOGGLE ) {
			return;
		}

		var ARIA_EXPANDED_VALUE = isToOpenSubMenu ? ATTRIBUTE_VALUE_TRUE : ATTRIBUTE_VALUE_FALSE;

		SUB_MENU_TOGGLE.setAttribute( ATTRIBUTE_NAME_ARIA_EXPANDED, ARIA_EXPANDED_VALUE );

		var SUB_MENU_TOGGLE_TEXT_OPEN = getElementByClassName( CLASS_NAV_SUB_MENU_TOGGLE_TEXT_OPEN, SUB_MENU_TOGGLE );
		var SUB_MENU_TOGGLE_TEXT_CLOSE = getElementByClassName( CLASS_NAV_SUB_MENU_TOGGLE_TEXT_CLOSE, SUB_MENU_TOGGLE );

		if ( SUB_MENU_TOGGLE_TEXT_OPEN ) {
			SUB_MENU_TOGGLE_TEXT_OPEN.setAttribute( ATTRIBUTE_NAME_ARIA_HIDDEN, isToOpenSubMenu );
		}

		if ( SUB_MENU_TOGGLE_TEXT_CLOSE ) {
			SUB_MENU_TOGGLE_TEXT_CLOSE.setAttribute( ATTRIBUTE_NAME_ARIA_HIDDEN, ! isToOpenSubMenu );
		}
	}

	var siteHeaderHeight;

	function addStickyHeaderEvents() {
		window.addEventListener( EVENT_TYPE_LOAD, function() {
			resetSiteContentPaddingTop();
		} );
		window.addEventListener( EVENT_TYPE_RESIZE, function() {
			resetSiteContentPaddingTop();
		} );
	}

	function initialiseSiteHeaderHeight() {
		var SITE_HEADER = getSiteHeader();

		if ( SITE_HEADER ) {
			siteHeaderHeight = window.getComputedStyle( SITE_HEADER, null ).height;
		}
	}

	function resetSiteContentPaddingTop() {
		initialiseSiteHeaderHeight();
		toggleSiteContentContainerPaddingTop();
	}

	function toggleSiteContentContainerPaddingTop() {
		var SITE_CONTENT_CONTAINER = getSiteContentContainer();
		var SITE_HEADER = getSiteHeader();

		if ( SITE_CONTENT_CONTAINER && SITE_HEADER ) {
			if ( isFixed( SITE_HEADER ) ) {
				SITE_CONTENT_CONTAINER.style.paddingTop = siteHeaderHeight;

				return;
			}

			removeStyle( SITE_CONTENT_CONTAINER );
		}
	}

	function addStickyMenuEvents() {
		window.addEventListener( EVENT_TYPE_LOAD, function() {
			toggleStickyMenu();
		} );
		window.addEventListener( EVENT_TYPE_RESIZE, function() {
			toggleStickyMenu();
		} );
		window.addEventListener( EVENT_TYPE_SCROLL, function() {
			toggleStickyMenu();
		} );
	}

	function addMarginBottomToSiteHeader( mainNavigation, siteHeader ) {
		if ( isMainNavigation( mainNavigation ) && isSiteHeader( siteHeader ) ) {
			siteHeader.style.marginBottom = window.getComputedStyle( mainNavigation, null ).height;
		}
	}

	function isMainNavigation( element ) {
		return isHTMLElement( element, CLASS_NAV_MAIN_NAVIGATION );
	}

	function isMainNavigationSticky( element ) {
		return isMainNavigation( element ) && matches( element, CLASS_SHARED_STICKY );
	}

	function isSiteHeader( element ) {
		return isHTMLElement( element, CLASS_SECTION_SITE_HEADER );
	}

	function mainNavigationHeightChanged( mainNavigation, siteHeader ) {
		return isMainNavigation( mainNavigation ) && isSiteHeader( siteHeader ) && siteHeader.style.marginBottom !== window.getComputedStyle( mainNavigation, null ).height;
	}

	function removeMarginBottomFromSiteHeader( element ) {
		if ( isSiteHeader( element ) ) {
			removeStyle( element );
		}
	}

	function toggleStickyMenu() {
		var SITE_HEADER = getSiteHeader();
		var MAIN_NAVIGATION = getMainNavigation();

		if ( MAIN_NAVIGATION && SITE_HEADER ) {
			if ( isFixed( SITE_HEADER ) ) {
				removeMarginBottomFromSiteHeader( SITE_HEADER );

				return;
			}

			if ( isMainNavigationSticky( MAIN_NAVIGATION ) ) {
				if ( mainNavigationHeightChanged( MAIN_NAVIGATION, SITE_HEADER ) ) {
					updateSiteHeaderMarginBottom( MAIN_NAVIGATION, SITE_HEADER );
				}

				if ( window.pageYOffset <= SITE_HEADER.offsetHeight ) {
					removeClass( MAIN_NAVIGATION, CLASS_SHARED_STICKY );
					removeMarginBottomFromSiteHeader( SITE_HEADER );
				}

				return;
			}

			if ( window.pageYOffset >= MAIN_NAVIGATION.offsetTop ) {
				addClass( MAIN_NAVIGATION, CLASS_SHARED_STICKY );
				addMarginBottomToSiteHeader( MAIN_NAVIGATION, SITE_HEADER );
			}
		}
	}

	function updateSiteHeaderMarginBottom( mainNavigation, siteHeader ) {
		if ( isMainNavigation( mainNavigation ) && isSiteHeader( siteHeader ) ) {
			addMarginBottomToSiteHeader( mainNavigation, siteHeader );
		}
	}

	addMenuEvents();
	addStickyMenuEvents();
	addStickyHeaderEvents();
}() );
