/**
 * Theme Name: errigal
 * Theme URI: https://projecterrigal.com/themes/errigal/
 * Author: Project errigal
 * Author URI: https://projecterrigal.com/
 * Version: 1.0.4
 */
( function() {
	'use strict';

	var ATTRIBUTE_NAME_STYLE = 'style';
	var CHARACTER_PERIOD = '.';
	var CLASS_SECTION_SITE = 'site';
	var CLASS_SECTION_SITE_CONTENT = 'site-content';
	var CLASS_SECTION_SITE_FOOTER = 'site-footer';
	var CLASS_SHARED_AUTOFOCUSED = 'autofocused';
	var EMPTY_STRING = '';
	var EVENT_TYPE_CLICK = 'click';
	var EVENT_TYPE_FOCUSIN = 'focusin';
	var EVENT_TYPE_FOCUSOUT = 'focusout';
	var EVENT_TYPE_LOAD = 'load';
	var EVENT_TYPE_RESIZE = 'resize';
	var NODE_TYPE_HTML_ELEMENT = 1;
	var NODE_TYPE_DOCUMENT = 9;
	var POSITION_FIXED = 'fixed';

	function addClass( element, className ) {
		if ( isHTMLElement( element ) && element.classList && isString( className ) && ! isNullOrEmpty( className ) ) {
			element.classList.add( className );
		}
	}

	function matches( element, selectorString, isTag ) {
		if ( ! isHTMLElement( element ) || ! isString( selectorString ) || isNullOrEmpty( selectorString ) ) {
			return false;
		}

		if ( ! sanitiseBoolean( isTag ) && selectorString.indexOf( CHARACTER_PERIOD ) !== 0 ) {
			selectorString = CHARACTER_PERIOD + selectorString;
		}

		return element.matches ? element.matches( selectorString ) : element.msMatchesSelector( selectorString );
	}

	function removeClass( element, className ) {
		if ( isHTMLElement( element ) && element.classList && isString( className ) && ! isNullOrEmpty( className ) ) {
			element.classList.remove( className );
		}
	}

	function removeStyle( element ) {
		if ( isHTMLElement( element ) ) {
			element.removeAttribute( ATTRIBUTE_NAME_STYLE );
		}
	}

	function getElementByClassName( className, rootElement ) {
		if ( ! isString( className ) || isNullOrEmpty( className ) ) {
			return null;
		}

		if ( ! rootElement ) {
			rootElement = document;
		}

		return isHTMLElement( rootElement ) ? rootElement.getElementsByClassName( className )[ 0 ] : null;
	}

	function sanitiseBoolean( value ) {
		return isBoolean( value ) ? value : false;
	}

	function isBoolean( value ) {
		return value === false || value === true;
	}

	function isHTMLElement( element, className ) {
		if ( ! element ) {
			return false;
		}

		var IS_HTML_ELEMENT = element.nodeType === NODE_TYPE_HTML_ELEMENT && element instanceof HTMLElement;
		var IS_DOCUMENT = element.nodeType === NODE_TYPE_DOCUMENT && element instanceof Document;
		var IS_HTML_ELEMENT_OR_DOCUMENT = IS_HTML_ELEMENT || IS_DOCUMENT;

		if ( isString( className ) && ! isNullOrEmpty( className ) ) {
			return IS_HTML_ELEMENT_OR_DOCUMENT && matches( element, className );
		}

		return IS_HTML_ELEMENT_OR_DOCUMENT;
	}

	function isNullOrEmpty( value ) {
		return value === null || value === undefined || value === EMPTY_STRING;
	}

	function isString( value ) {
		return typeof value === 'string';
	}

	function getAutoFocusedElements() {
		return document.querySelectorAll( '.'.concat( CLASS_SHARED_AUTOFOCUSED ) );
	}

	function getSite() {
		return getElementByClassName( CLASS_SECTION_SITE );
	}

	function getSiteContent() {
		return getElementByClassName( CLASS_SECTION_SITE_CONTENT );
	}

	function getSiteFooter() {
		return getElementByClassName( CLASS_SECTION_SITE_FOOTER );
	}

	var isClickAction = false;

	function addAccessibilityCommonIeEvents() {
		document.addEventListener( EVENT_TYPE_CLICK, function() {
			isClickAction = true;
		} );
		document.addEventListener( EVENT_TYPE_FOCUSIN, function( event ) {
			clearAutoFocusedElements();

			if ( isClickAction ) {
				addClass( event.target, CLASS_SHARED_AUTOFOCUSED );
				isClickAction = false;
			}
		} );
		document.addEventListener( EVENT_TYPE_FOCUSOUT, function() {
			clearAutoFocusedElements();
		} );
	}

	function clearAutoFocusedElements() {
		var AUTOFOCUSED_ELEMENTS = getAutoFocusedElements();

		for ( var i = 0; i < AUTOFOCUSED_ELEMENTS.length; i++ ) {
			removeClass( AUTOFOCUSED_ELEMENTS[ i ], CLASS_SHARED_AUTOFOCUSED );
		}
	}

	function addSiteContentEvents() {
		window.addEventListener( EVENT_TYPE_LOAD, function() {
			setSiteContentBottomMargin();
		} );
		window.addEventListener( EVENT_TYPE_RESIZE, function() {
			setSiteContentBottomMargin();
		} );
	}

	function setSiteContentBottomMargin() {
		var SITE = getSite();
		var SITE_CONTENT = getSiteContent();
		var SITE_FOOTER = getSiteFooter();

		if ( SITE && SITE_CONTENT && SITE_FOOTER ) {
			var SITE_CONTENT_RECT = SITE_CONTENT.getBoundingClientRect();
			var SITE_FOOTER_RECT = SITE_FOOTER.getBoundingClientRect();

			if ( ! SITE_CONTENT_RECT || ! SITE_FOOTER_RECT ) {
				return;
			}

			var SITE_CONTENT_BOTTOM_POSITION = SITE_CONTENT_RECT.bottom;
			var SITE_FOOTER_TOP_POSITION = window.innerHeight - SITE_FOOTER_RECT.height + parseInt( getComputedStyle( SITE_FOOTER, null ).marginTop );

			if ( SITE_CONTENT_BOTTOM_POSITION < SITE_FOOTER_TOP_POSITION ) {
				SITE_FOOTER.style.position = POSITION_FIXED;
				SITE_FOOTER.style.bottom = 0;
				SITE_FOOTER.style.left = 0;
				SITE_FOOTER.style.right = 0;

				return;
			}

			removeStyle( SITE_FOOTER );
		}
	}

	addAccessibilityCommonIeEvents();
	addSiteContentEvents();
}() );
