<?php
/**
 * Custom Base Class
 *
 * @package errigal
 * @since 1.0.0
 */

/**
 * This class is in charge of providing customiser shared settings and functions.
 *
 * @since 1.0.0
 */
abstract class Errigal_Custom_Base {
	/**
	 * Section: Pro features.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	protected const SECTION_PRO_FEATURES = 'pro_features';

	/**
	 * Section: Appearance.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	protected const SECTION_APPEARANCE = 'appearance';

	/**
	 * Section: Font settings.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	protected const SECTION_FONT_SETTINGS = 'font_settings';

	/**
	 * Section: Footer.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	protected const SECTION_FOOTER = 'footer';

	/**
	 * Section order priority: Pro features.
	 *
	 * @since 1.0.0
	 *
	 * @var int
	 */
	protected const SECTION_PRIORITY_PRO_FEATURES = 10;

	/**
	 * Section order priority: Appearance.
	 *
	 * @since 1.0.0
	 *
	 * @var int
	 */
	protected const SECTION_PRIORITY_APPEARANCE = 30;

	/**
	 * Section order priority: Font settings.
	 *
	 * @since 1.0.0
	 *
	 * @var int
	 */
	protected const SECTION_PRIORITY_FONT_SETTINGS = 31;

	/**
	 * Section order priority: Homepage Settings.
	 *
	 * @since 1.0.0
	 *
	 * @var int
	 */
	protected const SECTION_PRIORITY_HOMEPAGE_SETTINGS = 32;

	/**
	 * Control order priority in Appearance: Colour scheme.
	 *
	 * @since 1.0.0
	 *
	 * @var int
	 */
	protected const CONTROL_PRIORITY_APPEARANCE_COLOUR_SCHEME = 1;

	/**
	 * Control order priority in Appearance: Site header colour.
	 *
	 * @since 1.0.0
	 *
	 * @var int
	 */
	protected const CONTROL_PRIORITY_APPEARANCE_SITE_HEADER_COLOUR = 2;

	/**
	 * Control order priority in Appearance: Background colour.
	 *
	 * @since 1.0.0
	 *
	 * @var int
	 */
	protected const CONTROL_PRIORITY_APPEARANCE_BACKGROUND_COLOUR = 3;

	/**
	 * Sanitize callback: radio.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	protected const SANITIZE_CALLBACK_RADIO = array( __CLASS__, 'sanitize_radio' );

	/**
	 * Sanitize callback: select.
	 *
	 * @since 1.0.0
	 *
	 * @var array
	 */
	protected const SANITIZE_CALLBACK_SELECT = array( __CLASS__, 'sanitize_select' );

	/**
	 * Sanitize callback: textarea with HTML tags allowed.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	protected const SANITIZE_CALLBACK_TEXTAREA_HTML = 'wp_kses_post';

	/**
	 * Sanitization function for radio.
	 *
	 * @since 1.0.0
	 *
	 * @param string $input   Input value.
	 * @param object $setting Setting object.
	 * @return string
	 */
	public static function sanitize_radio( $input, $setting ) {
		$input   = sanitize_key( $input );
		$choices = $setting->manager->get_control( $setting->id )->choices;

		return array_key_exists( $input, $choices ) ? $input : $setting->default;
	}

	/**
	 * Sanitization function for select.
	 *
	 * @since 1.0.0
	 *
	 * @param string $input   Input value.
	 * @param object $setting Setting object.
	 * @return string
	 */
	public static function sanitize_select( $input, $setting ) {
		$input   = sanitize_key( $input );
		$choices = $setting->manager->get_control( $setting->id )->choices;

		return array_key_exists( $input, $choices ) ? $input : $setting->default;
	}
}
