<?php
/**
 * Custom Pro Features Class
 *
 * @package errigal
 * @since 1.0.0
 */

/**
 * This class is in charge of showing pro features in customiser.
 *
 * @since 1.0.0
 *
 * @see Errigal_Custom_Base
 */
final class Errigal_Custom_Pro_Features extends Errigal_Custom_Base {
	/**
	 * Setting ID: Pro features.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	private const ID_PRO_FEATURES = 'pro_features';

	/**
	 * Constructor. Instantiates the object.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		add_action( 'customize_register', array( $this, 'register' ) );
	}

	/**
	 * Registers customiser options.
	 *
	 * @since 1.0.0
	 *
	 * @see Errigal_Customise_Pro_Features_Control
	 *
	 * @param WP_Customize_Manager $wp_customise Theme customiser object.
	 * @return void
	 */
	public function register( $wp_customise ) {
		$wp_customise->add_setting(
			self::ID_PRO_FEATURES,
			array( 'sanitize_callback' => 'wp_filter_nohtml_kses' )
		);

		$wp_customise->add_control(
			new Errigal_Customise_Pro_Features_Control(
				$wp_customise,
				self::ID_PRO_FEATURES,
				array(
					'section' => self::SECTION_PRO_FEATURES,
				)
			)
		);
	}
}

new Errigal_Custom_Pro_Features();
