<?php
/**
 * Customize API: Errigal_Customise_Pro_Features_Control Class
 *
 * @package errigal
 * @since 1.0.0
 */

if ( class_exists( 'WP_Customize_Control' ) ) :
	/**
	 * Customise Pro Features Control Class.
	 *
	 * @since 1.0.0
	 *
	 * @see WP_Customize_Control
	 */
	final class Errigal_Customise_Pro_Features_Control extends WP_Customize_Control {
		/**
		 * Control type.
		 *
		 * @since 1.0.0
		 *
		 * @var string
		 */
		public $type = 'errigal-pro-features';

		/**
		 * Renders content of custom pro features.
		 *
		 * @since 1.0.0
		 *
		 * @return void
		 */
		public function render_content() {
			?>
			<div class="pro-features">
				<?php
				$this->render_pro_features_list();

				$this->render_discover_button();
				?>
			</div>
			<?php
		}

		/**
		 * Renders pro features list.
		 *
		 * @since 1.0.0
		 *
		 * @return void
		 */
		public function render_pro_features_list() {
			$priority_pro_features = $this->get_priority_pro_features();
			$pro_features          = $this->get_pro_features();

			if ( empty( $pro_features ) ) {
				return;
			}

			?>
			<ul>
				<?php
				foreach ( $priority_pro_features as $priority_pro_feature ) {
					printf(
						wp_kses_post( '<li><strong>%s</strong></li>' ),
						esc_html( $priority_pro_feature )
					);
				}

				foreach ( $pro_features as $pro_feature ) {
					printf(
						wp_kses_post( '<li>%s</li>' ),
						esc_html( $pro_feature )
					);
				}
				?>
			</ul>
			<?php
		}

		/**
		 * Renders discover button.
		 *
		 * @since 1.0.0
		 *
		 * @return void
		 */
		public function render_discover_button() {
			$author_uri = wp_get_theme()->get( 'AuthorURI' );
			$theme_name = wp_get_theme()->get( 'Name' );

			if ( empty( $author_uri ) || empty( $theme_name ) ) {
				return;
			}

			printf(
				wp_kses_post( '<a class="button" href="%s" rel="noopener" target="_blank">%s</a>' ),
				esc_url(
					sprintf(
						'%sthemes/%s-pro/',
						esc_url( $author_uri ),
						esc_attr( $theme_name )
					)
				),
				esc_html__( 'Discover errigal PRO', 'errigal' )
			);
		}

		/**
		 * Gets priority pro features.
		 *
		 * @since 1.0.0
		 *
		 * @return array
		 */
		private function get_priority_pro_features() {
			return array(
				esc_html__( 'Hero banner carousel', 'errigal' ),
				esc_html__( 'Table of contents', 'errigal' ),
				esc_html__( 'Cookie notification', 'errigal' ),
				esc_html__( 'Share buttons', 'errigal' ),
				esc_html__( 'Related posts', 'errigal' ),
			);
		}

		/**
		 * Gets pro features.
		 *
		 * @since 1.0.0
		 *
		 * @return array
		 */
		private function get_pro_features() {
			return array(
				esc_html__( 'Sidebar layout options', 'errigal' ),
				esc_html__( 'Multiple blog header styles', 'errigal' ),
				esc_html__( 'Additional colour schemes', 'errigal' ),
				esc_html__( 'and more...', 'errigal' ),
			);
		}
	}
endif;
