<?php
/**
 * Social Links Class
 *
 * @package errigal
 * @since 1.0.3
 */

/**
 * This class is in charge of displaying social links.
 *
 * @since 1.0.0
 */
class Errigal_Social_Links {
	/**
	 * Gets social links.
	 *
	 * @since 1.0.3
	 *
	 * @param array $args {
	 *     Optional. Social link options.
	 *
	 *     @type string $icon_size     Icon-size of sm, base, md, lg, xl, 2xl or 3xl. Default md.
	 *     @type string $facebook_url  Facebook URL.
	 *     @type string $twitter_url   Twitter URL.
	 *     @type string $instagram_url Instagram URL.
	 *     @type string $linkedin_url  LinkedIn URL.
	 *     @type string $youtube_url   YouTube URL.
	 *     @type string $line_url      LINE URL.
	 *     @type string $github_url    GitHub URL.
	 * @return string
	 */
	public static function get_social_links( $args = array() ) {
		$args = wp_parse_args(
			$args,
			array(
				'icon_size'     => 'md',
				'facebook_url'  => '',
				'twitter_url'   => '',
				'instagram_url' => '',
				'linkedin_url'  => '',
				'youtube_url'   => '',
				'line_url'      => '',
				'github_url'    => '',
			)
		);

		$icon_size = $args['icon_size'];

		return self::get_social_link( $args['facebook_url'], esc_html__( 'Facebook', 'errigal' ), errigal_get_icon_svg( 'facebook', $icon_size ) ) .
			self::get_social_link( $args['twitter_url'], esc_html__( 'Twitter', 'errigal' ), errigal_get_icon_svg( 'twitter', $icon_size ) ) .
			self::get_social_link( $args['instagram_url'], esc_html__( 'Instagram', 'errigal' ), errigal_get_icon_svg( 'instagram', $icon_size ) ) .
			self::get_social_link( $args['linkedin_url'], esc_html__( 'LinkedIn', 'errigal' ), errigal_get_icon_svg( 'linkedin', $icon_size ) ) .
			self::get_social_link( $args['youtube_url'], esc_html__( 'YouTube', 'errigal' ), errigal_get_icon_svg( 'youtube', $icon_size ) ) .
			self::get_social_link( $args['line_url'], esc_html__( 'LINE', 'errigal' ), errigal_get_icon_svg( 'line', $icon_size ) ) .
			self::get_social_link( $args['github_url'], esc_html__( 'GitHub', 'errigal' ), errigal_get_icon_svg( 'github', $icon_size ) );
	}

	/**
	 * Gets social link.
	 *
	 * @since 1.0.3
	 *
	 * @param string $social_link_url   Social link URL.
	 * @param string $social_media_name Social media name.
	 * @param string $social_icon       Social icon.
	 * @return string
	 */
	protected static function get_social_link( $social_link_url, $social_media_name, $social_icon ) {
		if ( empty( $social_link_url ) || empty( $social_media_name ) || empty( $social_icon ) ) {
			return '';
		}

		return sprintf(
			'<a class="social-icon-link" href="%s" target="_blank" aria-label="%s">%s</a>',
			esc_url( $social_link_url ),
			esc_attr( $social_media_name ),
			wp_kses( $social_icon, errigal_get_allowed_html_svg() )
		);
	}
}
