<?php
/**
 * Template for displaying comments
 *
 * This is the template that displays area of page that contains both current comments
 * and comment form.
 *
 * @link https://developer.wordpress.org/themes/basics/template-hierarchy/
 *
 * @package errigal
 * @since 1.0.0
 */

/*
 * Returns early without loading comments, if current post is protected by password
 * and visitor has not yet entered password.
 */
if ( post_password_required() ) {
	return;
}
?>

<div id="comments" class="post-comments">

	<?php
	if ( have_comments() ) :
		?>
		<h2 class="comments-title">
			<?php
			$errigal_comment_count = get_comments_number();

			if ( '1' === $errigal_comment_count ) {
				echo esc_html__( '1 comment', 'errigal' );
			} else {
				printf(
					/* translators: 1: Comment count number. */
					esc_html( _nx( '%1$s comment', '%1$s comments', $errigal_comment_count, 'Comments title', 'errigal' ) ),
					esc_html( number_format_i18n( $errigal_comment_count ) )
				);
			}
			?>
		</h2><!-- .comments-title -->

		<ol class="comment-list<?php echo get_option( 'show_avatars' ) === '1' ? ' has-avatar' : ''; ?>">
			<?php
			wp_list_comments(
				array(
					'style'      => 'ol',
					'short_ping' => true,
				)
			);
			?>
		</ol><!-- .comment-list -->

		<?php
		the_comments_pagination(
			array(
				'mid_size'  => 0,
				'prev_text' => sprintf(
					'%s<span class="screen-reader-text">%s</span>',
					wp_kses( errigal_get_icon_svg( 'previous' ), errigal_get_allowed_html_svg() ),
					esc_html__( 'Older comments', 'errigal' )
				),
				'next_text' => sprintf(
					'%s<span class="screen-reader-text">%s</span>',
					wp_kses( errigal_get_icon_svg( 'next' ), errigal_get_allowed_html_svg() ),
					esc_html__( 'Newer comments', 'errigal' )
				),
			)
		);

		// If comments exists but are closed, prints message saying comments are closed.
		if ( ! comments_open() ) :
			?>
			<p class="no-comments"><?php esc_html_e( 'Comments are closed.', 'errigal' ); ?></p>
			<?php
		endif; // Check for comments_open().

	endif; // Check for have_comments().

	comment_form(
		array(
			'logged_in_as' => null,
			'class_submit' => 'button contained',
			'title_reply'  => esc_html__( 'Leave a comment', 'errigal' ),
		)
	);
	?>

</div><!-- #comments -->
