<?php
/**
 * Custom template tags for this theme
 *
 * Eventually, some of the functionality here could be replaced by core features.
 *
 * @package errigal
 * @since 1.0.0
 */

if ( ! function_exists( 'errigal_nav_menu' ) ) :
	/**
	 * Renders HTML with nav menu.
	 *
	 * @since 1.0.0
	 *
	 * @return void
	 */
	function errigal_nav_menu() {
		get_template_part( 'template-parts/component/main-menu' );
	}
endif;

if ( ! function_exists( 'errigal_single_post' ) ) :
	/**
	 * Renders HTML with post for single.php.
	 *
	 * @since 1.0.0
	 *
	 * @return void
	 */
	function errigal_single_post() {
		get_template_part( 'template-parts/content/content-single' );
	}
endif;

if ( ! function_exists( 'errigal_single_post_header' ) ) :
	/**
	 * Renders HTML with post header for single.php.
	 *
	 * @since 1.0.0
	 *
	 * @return void
	 */
	function errigal_single_post_header() {
		get_template_part( 'template-parts/header/post-header' );
	}
endif;

if ( ! function_exists( 'errigal_the_post_thumbnail' ) ) :
	/**
	 * Renders HTML with post thumbnail.
	 *
	 * @since 1.0.0
	 *
	 * @global bool $is_IE Whether browser is Internet Explorer.
	 *
	 * @param string $class Class for <figure> element.
	 * @param array  $args {
	 *     Optional. Formatting options.
	 *
	 *     @type int|WP_Post  $post         Post ID or WP_Post object.  Default is global `$post`.
	 *     @type string|int[] $size         Image size. Accepts any registered image size name, or an array of
	 *                                      width and height values in pixels (in that order). Default 'post-thumbnail'.
	 *     @type string|false $loading      The 'loading' attribute value. Passing a value of false
	 *                                      will result in the attribute being omitted for the image.
	 *                                      Defaults to 'lazy', depending on wp_lazy_loading_enabled().
	 *     @type bool         $require_ie   Whether to use IE workaround. Default true.
	 *     @type bool         $include_link Whether to include link to post. Default true.
	 *     @type bool         $echo         Whether to echo or return thumbnail. Default true.
	 * }
	 * @return void|string
	 */
	function errigal_the_post_thumbnail( $class, $args = array() ) {
		global $is_IE;

		$args = wp_parse_args(
			$args,
			array(
				'post'         => null,
				'size'         => null,
				'loading'      => false,
				'require_ie'   => true,
				'include_link' => true,
				'echo'         => true,
			)
		);

		$post = $args['post'];

		$can_show_post_thumbnail =
			! post_password_required( $post ) &&
			! is_attachment() &&
			has_post_thumbnail( $post );

		if ( ! $can_show_post_thumbnail || empty( $class ) ) {
			return '';
		}

		$image_class = 'post-thumbnail';
		$size        = $args['size'];
		$loading     = $args['loading'];

		$the_post_thumbnail = $is_IE && $args['require_ie'] ?
			sprintf(
				wp_kses_post( '<div class="%1$s" style="background-image: url(%2$s);"></div>' ),
				esc_attr( $image_class ),
				esc_url(
					get_the_post_thumbnail_url(
						$post,
						$size,
						array( 'loading' => $loading )
					)
				)
			) :
			get_the_post_thumbnail(
				$post,
				$size,
				array(
					'class'   => $image_class,
					'loading' => $loading,
				)
			);

		$output = sprintf(
			wp_kses_post( '<figure class="%1$s">%2$s</figure>' ),
			esc_attr( $class ),
			$args['include_link'] ?
				sprintf(
					'<a href="%1$s" aria-label="%2$s">%3$s</a>',
					esc_url( get_the_permalink() ),
					esc_attr( get_the_title() ),
					wp_kses_post( $the_post_thumbnail )
				) :
				wp_kses_post( $the_post_thumbnail )
		);

		if ( $args['echo'] ) {
			echo wp_kses_post( $output );
		} else {
			return $output;
		}
	}
endif;
