<?php
/**
 * Template for displaying search forms
 *
 * @link https://developer.wordpress.org/reference/functions/wp_unique_id/
 * @link https://developer.wordpress.org/reference/functions/get_search_form/
 *
 * @package errigal
 * @since 1.0.0
 */

/*
 * Generates an unique ID for each form and a string containing an aria-label
 * if one was passed to get_search_form() in the args array.
 */
$errigal_unique_id = wp_unique_id( 'search-form-' );

$errigal_aria_label = ! empty( $args['aria_label'] ) ? 'aria-label="' . esc_attr( $args['aria_label'] ) . '"' : '';

$search_query             = get_search_query();
$clear_button_state_class = empty( $search_query ) ? '' : ' show';
?>

<form role="search" <?php echo $errigal_aria_label; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped -- Escaped above. ?> method="get" class="search-form" action="<?php echo esc_url( home_url( '/' ) ); ?>">
	<label class="screen-reader-text" for="<?php echo esc_attr( $errigal_unique_id ); ?>"><?php echo esc_html__( 'Search&hellip;', 'errigal' ); ?></label>
	<div class="search-field-container">
		<input required type="text" id="<?php echo esc_attr( $errigal_unique_id ); ?>" class="search-field" placeholder="<?php echo esc_attr__( 'Search&hellip;', 'errigal' ); ?>" value="<?php echo esc_html( $search_query ); ?>" name="s" />
		<button class="search-clear-button<?php echo esc_attr( $clear_button_state_class ); ?>" aria-label="<?php esc_attr_e( 'Clear keyword', 'errigal' ); ?>">
			<?php errigal_render_icon_svg( 'times' ); ?>
		</button>
	</div>
	<button type="submit" class="search-submit" aria-label="<?php esc_attr_e( 'Search', 'errigal' ); ?>">
		<?php errigal_render_icon_svg( 'search', 'md' ); ?>
	</button>
</form>
