<?php
/**
 * Template part for displaying profile widget
 *
 * @package errigal
 * @since 1.0.3
 *
 * @param array $args {
 *     Profile widget options.
 *
 *     @type string $profile_page_id Post ID for profile page.
 *     @type string $image_url       Image URL.
 *     @type string $image_alt       Image alternate text.
 *     @type string $name            Profile name.
 *     @type string $description     Profile description.
 *     @type string $facebook_url    Facebook URL.
 *     @type string $twitter_url     Twitter URL.
 *     @type string $instagram_url   Instagram URL.
 *     @type string $linkedin_url    LinkedIn URL.
 *     @type string $youtube_url     YouTube URL.
 *     @type string $line_url        LINE URL.
 *     @type string $github_url      GitHub URL.
 * }
 */

$args = wp_parse_args(
	$args,
	array(
		'profile_page_id' => -1,
		'image_url'       => '',
		'image_alt'       => '',
		'name'            => '',
		'description'     => '',
		'facebook_url'    => '',
		'twitter_url'     => '',
		'instagram_url'   => '',
		'linkedin_url'    => '',
		'youtube_url'     => '',
		'line_url'        => '',
		'github_url'      => '',
	)
);

$profile_page_id = $args['profile_page_id'];
$image_url       = $args['image_url'];
$image_alt       = $args['image_alt'];
$name            = $args['name'];
$description     = $args['description'];

$has_profile_page_link = ! empty( $profile_page_id ) && is_int( $profile_page_id ) && $profile_page_id > 0;
?>

<div class='errigal-profile-widget__container'>
	<figure class="errigal-profile-widget__profile-image-container">
		<?php
		if ( $has_profile_page_link ) {
			?>
			<a href="<?php echo esc_url( get_permalink( $profile_page_id ) ); ?>" aria-label="<?php echo esc_attr( $image_alt ); ?>">
			<?php
		}

		if ( empty( $image_url ) ) {
			errigal_render_icon_svg( 'default-profile-image', '3xl' );
		} else {
			get_template_part(
				'template-parts/component/widget/image',
				null,
				array(
					'image_class' => 'errigal-profile-widget__profile-image',
					'image_url'   => esc_url( $image_url ),
					'image_alt'   => esc_attr( $image_alt ),
				)
			);
		}

		if ( $has_profile_page_link ) {
			?>
			</a>
			<?php
		}
		?>
	</figure>
	<?php
	if ( ! empty( $name ) ) {
		?>
		<div class="errigal-profile-widget__profile-name"><?php echo esc_html( $name ); ?></div>
		<?php
	}

	if ( ! empty( $description ) ) {
		?>
		<div class="errigal-profile-widget__description"><?php echo wp_kses_post( $description ); ?></div>
		<?php
	}

	$social_links = errigal_get_social_links(
		array(
			'facebook_url'  => $args['facebook_url'],
			'twitter_url'   => $args['twitter_url'],
			'instagram_url' => $args['instagram_url'],
			'linkedin_url'  => $args['linkedin_url'],
			'youtube_url'   => $args['youtube_url'],
			'line_url'      => $args['line_url'],
			'github_url'    => $args['github_url'],
		)
	);

	if ( ! empty( $social_links ) ) {
		?>
		<div class="errigal-profile-widget__social-icons-container">
			<?php echo wp_kses( $social_links, errigal_get_allowed_html_post_svg() ); ?>
		</div>
		<?php
	}
	?>
</div>
